"""日志工具模块

提供实用工具函数，如禁用/启用日志记录、拦截标准库日志等。
"""

import logging
import sys
import inspect
import os
import time
import uuid
import threading
import contextlib
from functools import wraps
from types import ModuleType
from typing import Optional, Dict, Any, List, Generator

from loguru import logger


# 请求上下文数据的线程本地存储
_request_context = threading.local()


def disable_logger(level: str = "INFO") -> None:
    """禁用特定级别及以下的日志记录

    Args:
        level: 要禁用的日志级别，默认为INFO
    """
    logger.disable(None)
    logger.enable(level)


def enable_logger() -> None:
    """启用所有日志记录"""
    logger.configure(handlers=[])
    logger.enable(None)


def reset_logger() -> None:
    """重置日志记录器配置"""
    logger.remove()


class LoggingInterceptHandler(logging.Handler):
    """拦截标准库日志并重定向到loguru的处理器"""

    def emit(self, record: logging.LogRecord) -> None:
        # 获取对应的loguru级别，默认为同名级别
        level = record.levelname
        frame = inspect.currentframe()
        depth = 2  # 默认深度

        # 传递数据到loguru
        while frame and depth > 0:
            frame = frame.f_back
            depth -= 1

        # 提取frame信息，或使用默认值
        file_path = record.pathname if frame is None else frame.f_code.co_filename
        function = record.funcName if frame is None else frame.f_code.co_name
        line = record.lineno if frame is None else frame.f_lineno

        # 从logging记录中提取模块名作为source
        module = record.module
        source = getattr(record, "source", module)

        # 确保source不为空
        if not source or source == "root":
            source = "Logging"

        # 获取当前请求ID（如果有）
        request_id = getattr(_request_context, "request_id", None)
        extra_kwargs = {
            "source": source,
            "file": file_path,
            "line": line,
            "function": function,
        }
        if request_id:
            extra_kwargs["request_id"] = request_id

        logger.bind(**extra_kwargs).opt(depth=0).log(level, record.getMessage())


def intercept_logging(level: Optional[str] = None) -> None:
    """拦截标准库日志，重定向到loguru

    Args:
        level: 日志级别，默认为None（使用root logger的级别）
    """
    # 获取根日志记录器
    logging_logger = logging.getLogger()

    # 如果指定了级别，则设置
    if level is not None:
        logging_logger.setLevel(getattr(logging, level))

    # 移除所有现有处理器
    if logging_logger.handlers:
        for handler in logging_logger.handlers[:]:
            logging_logger.removeHandler(handler)

    # 添加拦截处理器
    intercept_handler = LoggingInterceptHandler()
    logging_logger.addHandler(intercept_handler)


def log_function_call(logger=None, level="DEBUG"):
    """记录函数调用的装饰器

    Args:
        logger: 日志记录器，默认使用loguru.logger
        level: 日志级别，默认为DEBUG

    Returns:
        装饰函数的装饰器
    """
    if logger is None:
        logger = logger

    def decorator(func):
        name = func.__name__

        @wraps(func)
        def wrapper(*args, **kwargs):
            logger_ = logger.bind(source=f"{func.__module__}.{name}")
            signature = ", ".join(
                [repr(a) for a in args] + [f"{k}={repr(v)}" for k, v in kwargs.items()]
            )

            # 限制签名长度，避免日志过长
            if len(signature) > 300:
                signature = signature[:150] + "..." + signature[-150:]

            logger_.log(level, f"调用: {name}({signature})")

            try:
                result = func(*args, **kwargs)

                # 记录结果，但限制结果长度
                result_repr = repr(result)
                if len(result_repr) > 300:
                    result_repr = result_repr[:150] + "..." + result_repr[-150:]

                logger_.log(level, f"返回: {name} -> {result_repr}")
                return result
            except Exception as e:
                logger_.error(f"异常: {name} -> {type(e).__name__}: {e}")
                raise

        return wrapper

    return decorator


class RequestContext:
    """请求上下文管理器

    用于创建和管理请求追踪上下文，提供请求ID和分布式追踪支持。
    可以作为上下文管理器使用，或直接调用方法。

    示例:
        # 使用上下文管理器
        with RequestContext() as ctx:
            logger.info("在请求上下文中记录日志")

        # 直接使用
        RequestContext.set_request_id("custom-id")
        logger.info("使用自定义请求ID记录日志")
        RequestContext.clear()
    """

    # 追踪相关的请求头
    TRACE_HEADERS = {
        "x-request-id": "request_id",
        "x-trace-id": "trace_id",
        "x-span-id": "span_id",
        "x-parent-id": "parent_id",
    }

    @classmethod
    def generate_request_id(cls) -> str:
        """生成唯一的请求ID

        Returns:
            唯一请求ID
        """
        return str(uuid.uuid4())

    @classmethod
    def get_request_id(cls) -> Optional[str]:
        """获取当前请求ID

        Returns:
            当前请求ID，如果不存在则返回None
        """
        return getattr(_request_context, "request_id", None)

    @classmethod
    def set_request_id(cls, request_id: Optional[str] = None) -> str:
        """设置请求ID

        Args:
            request_id: 请求ID，如果为None则自动生成

        Returns:
            设置的请求ID
        """
        if request_id is None:
            request_id = cls.generate_request_id()

        _request_context.request_id = request_id
        return request_id

    @classmethod
    def get_trace_data(cls) -> Dict[str, Any]:
        """获取当前追踪数据

        Returns:
            包含所有追踪数据的字典
        """
        data = {}
        for key in cls.TRACE_HEADERS.values():
            value = getattr(_request_context, key, None)
            if value:
                data[key] = value

        # 添加请求计时信息
        if hasattr(_request_context, "request_start_time"):
            start_time = getattr(_request_context, "request_start_time")
            data["request_start_time"] = start_time
            data["request_duration"] = time.time() - start_time

        return data

    @classmethod
    def extract_from_headers(cls, headers: Dict[str, str]) -> Dict[str, str]:
        """从HTTP请求头中提取追踪信息

        Args:
            headers: HTTP请求头

        Returns:
            追踪信息字典
        """
        trace_data = {}

        # 处理标准追踪头
        for header, attr in cls.TRACE_HEADERS.items():
            if header in headers:
                trace_data[attr] = headers[header]

        # 如果没有请求ID但有追踪ID，使用追踪ID作为请求ID
        if "request_id" not in trace_data and "trace_id" in trace_data:
            trace_data["request_id"] = trace_data["trace_id"]

        # 如果仍然没有请求ID，生成一个
        if "request_id" not in trace_data:
            trace_data["request_id"] = cls.generate_request_id()

        return trace_data

    @classmethod
    def set_trace_data(cls, trace_data: Dict[str, Any]) -> None:
        """设置追踪数据

        Args:
            trace_data: 追踪数据字典
        """
        for key, value in trace_data.items():
            setattr(_request_context, key, value)

    @classmethod
    def clear(cls) -> None:
        """清除当前追踪上下文"""
        for key in list(vars(_request_context).keys()):
            delattr(_request_context, key)

    def __init__(
        self, request_id: Optional[str] = None, headers: Dict[str, str] = None
    ):
        """初始化请求上下文

        Args:
            request_id: 请求ID，如果为None则自动生成
            headers: HTTP请求头，用于提取追踪信息
        """
        self.generated_request_id = False

        if headers:
            # 从请求头提取追踪信息
            trace_data = self.extract_from_headers(headers)
            if request_id:
                trace_data["request_id"] = request_id
            self.trace_data = trace_data
        else:
            # 使用指定的请求ID或生成新的
            self.trace_data = {"request_id": request_id or self.generate_request_id()}
            self.generated_request_id = request_id is None

        # 添加请求开始时间
        self.trace_data["request_start_time"] = time.time()

    def __enter__(self) -> "RequestContext":
        """进入上下文时设置追踪数据"""
        self.set_trace_data(self.trace_data)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        """退出上下文时清除追踪数据"""
        self.clear()


class BatchingSink:
    """批处理日志接收器

    将日志消息存储在内存缓冲区中，当达到指定的批处理大小或
    时间间隔时触发写入操作，减少I/O操作次数。

    Args:
        sink: 目标接收器（文件、函数等）
        batch_size: 批处理大小
        flush_interval: 刷新间隔（秒）
    """

    def __init__(
        self,
        sink: Any,
        batch_size: int = 100,
        flush_interval: float = 1.0,
    ):
        self.sink = sink
        self.batch_size = batch_size
        self.flush_interval = flush_interval
        self.buffer: List[str] = []
        self.lock = threading.Lock()
        self.last_flush_time = time.time()

        # 启动定时刷新线程
        if flush_interval > 0:
            self.flush_thread = threading.Thread(
                target=self._timed_flush_worker,
                daemon=True,
            )
            self.flush_thread.start()
        else:
            self.flush_thread = None

        # 注册程序退出时的刷新
        import atexit

        atexit.register(self.flush)

    def __call__(self, message: str) -> None:
        """接收日志消息

        Args:
            message: 日志消息
        """
        with self.lock:
            self.buffer.append(message)

            # 如果达到批处理大小，执行刷新
            if len(self.buffer) >= self.batch_size:
                self._flush()

    def _flush(self) -> None:
        """内部刷新方法，必须在获取锁的情况下调用"""
        if not self.buffer:
            return

        # 如果接收器是可调用对象，则逐条传递消息
        if callable(self.sink):
            for message in self.buffer:
                self.sink(message)
        # 如果是文件类对象，则一次性写入所有消息
        elif hasattr(self.sink, "write"):
            for message in self.buffer:
                self.sink.write(message)
            if hasattr(self.sink, "flush"):
                self.sink.flush()

        # 清空缓冲区
        self.buffer.clear()
        self.last_flush_time = time.time()

    def flush(self) -> None:
        """手动刷新缓冲区"""
        with self.lock:
            self._flush()

    def _timed_flush_worker(self) -> None:
        """定时刷新工作线程"""
        while True:
            time.sleep(
                min(0.1, self.flush_interval)
            )  # 每0.1秒检查一次，最多不超过刷新间隔

            current_time = time.time()
            if current_time - self.last_flush_time >= self.flush_interval:
                self.flush()


def traced_function(name: Optional[str] = None, logger=None, level="INFO"):
    """跟踪函数执行并记录日志的装饰器

    用于跟踪函数执行时间并在请求上下文中记录日志。
    如果函数在请求上下文中执行，会继承该上下文。

    Args:
        name: 操作名称，默认为函数名
        logger: 日志记录器，默认使用loguru.logger
        level: 日志级别，默认为INFO

    Returns:
        装饰函数的装饰器
    """
    if logger is None:
        logger = logger

    def decorator(func):
        func_name = name or func.__name__

        @wraps(func)
        def wrapper(*args, **kwargs):
            # 获取或生成请求ID
            request_id = RequestContext.get_request_id()
            if not request_id:
                # 不在请求上下文中，创建临时上下文
                with RequestContext():
                    return _traced_execution(
                        func, func_name, logger, level, args, kwargs
                    )
            else:
                # 已在请求上下文中，直接执行
                return _traced_execution(func, func_name, logger, level, args, kwargs)

        return wrapper

    return decorator


def _traced_execution(func, func_name, logger, level, args, kwargs):
    """执行被跟踪的函数并记录日志"""
    start_time = time.time()
    log_context = {
        "operation": func_name,
        "request_id": RequestContext.get_request_id(),
    }

    logger.bind(**log_context).log(level, f"开始执行 {func_name}")

    try:
        result = func(*args, **kwargs)

        # 计算执行时间
        execution_time = time.time() - start_time
        log_context["execution_time"] = f"{execution_time:.6f}s"

        logger.bind(**log_context).log(
            level, f"完成执行 {func_name} (耗时: {execution_time:.6f}s)"
        )
        return result
    except Exception as e:
        # 计算执行时间
        execution_time = time.time() - start_time
        log_context["execution_time"] = f"{execution_time:.6f}s"
        log_context["error"] = str(e)
        log_context["error_type"] = type(e).__name__

        logger.bind(**log_context).exception(
            f"执行 {func_name} 失败 (耗时: {execution_time:.6f}s): {str(e)}"
        )
        raise


@contextlib.contextmanager
def log_group(
    name: str, logger=None, level="INFO", **context
) -> Generator[None, None, None]:
    """创建一个日志分组，用于对相关日志进行分组

    Args:
        name: 分组名称
        logger: 日志记录器，默认使用loguru.logger
        level: 日志级别，默认为INFO
        **context: 附加的上下文信息

    Yields:
        无
    """
    if logger is None:
        logger = logger

    # 获取或使用现有请求ID
    request_id = RequestContext.get_request_id()
    with_context = request_id is None

    try:
        # 如果没有请求上下文，创建一个临时的
        if with_context:
            ctx = RequestContext()
            ctx.__enter__()

        # 记录分组开始日志
        log_ctx = {"group": name, **context}
        if request_id:
            log_ctx["request_id"] = request_id

        logger.bind(**log_ctx).log(level, f"开始: {name}")
        start_time = time.time()

        yield

        # 记录分组结束日志
        duration = time.time() - start_time
        log_ctx["duration"] = f"{duration:.6f}s"
        logger.bind(**log_ctx).log(level, f"结束: {name} (耗时: {duration:.6f}s)")

    finally:
        # 清理临时请求上下文
        if with_context:
            ctx.__exit__(None, None, None)


# 用于记录日志调用源的插件路径识别
class PluginLoggerProxy:
    """代理loguru.logger对象，自动标记插件来源"""

    def __init__(self, original_logger):
        self._original_logger = original_logger
        # 保存一个缓存，避免每次调用都要重新检测
        self._source_cache = {}

    def _detect_plugin_name(self):
        """从调用栈中检测插件名称"""
        frame = inspect.currentframe()

        try:
            # 首先，看看当前帧的代码是否在调用栈中
            code_id = id(frame.f_code)
            if code_id in self._source_cache:
                return self._source_cache[code_id]

            # 向上遍历调用栈，查找插件来源
            for _ in range(30):  # 增加搜索深度以确保找到正确的插件调用
                if frame is None:
                    break

                module_name = frame.f_globals.get("__name__", "")
                filename = frame.f_code.co_filename

                # 检查是否可以从局部变量中找到self，并获取类名
                if "self" in frame.f_locals:
                    self_obj = frame.f_locals["self"]
                    if hasattr(self_obj, "__class__"):
                        class_name = self_obj.__class__.__name__
                        module_path = getattr(self_obj.__class__, "__module__", "")

                        # 首先检查是否是插件类的实例
                        if class_name.endswith("Plugin"):
                            source = f"Plugins.{class_name}"
                            # 缓存结果
                            self._source_cache[code_id] = source
                            return source
                        # 其次检查模块是否来自插件目录
                        elif module_path.startswith("plugins."):
                            parts = module_path.split(".")
                            if len(parts) > 1 and parts[1] != "utils":
                                plugin_name = parts[1]
                                source = f"Plugins.{plugin_name}"
                                # 缓存结果
                                self._source_cache[code_id] = source
                                return source

                # 检查是否是插件目录中的文件
                if "plugins/" in filename or "plugins\\" in filename:
                    # 从文件路径中提取插件名称
                    plugin_parts = filename.split(os.path.sep)
                    try:
                        # 查找plugins目录索引
                        plugins_idx = plugin_parts.index("plugins")
                        # 取下一个部分作为插件名
                        if plugins_idx + 1 < len(plugin_parts):
                            plugin_name = plugin_parts[plugins_idx + 1]
                            # 避免将utils目录识别为插件
                            if plugin_name != "utils" and not plugin_name.startswith(
                                "__"
                            ):
                                source = f"Plugins.{plugin_name}"
                                # 缓存结果
                                self._source_cache[code_id] = source
                                return source
                    except ValueError:
                        pass  # plugins不在路径中

                # 或者从模块名称中提取
                if module_name.startswith("plugins."):
                    parts = module_name.split(".")
                    if len(parts) > 1:
                        plugin_name = parts[1]
                        # 避免utils模块被识别为插件
                        if plugin_name != "utils" and not plugin_name.startswith("__"):
                            source = f"Plugins.{plugin_name}"
                            # 缓存结果
                            self._source_cache[code_id] = source
                            return source

                # 检查调用方所在的模块是否定义了__plugin_name__
                plugin_name = frame.f_globals.get("__plugin_name__")
                if plugin_name and isinstance(plugin_name, str):
                    source = f"Plugins.{plugin_name}"
                    # 缓存结果
                    self._source_cache[code_id] = source
                    return source

                frame = frame.f_back

            # 如果没能确定具体插件，但确定是从插件目录调用的
            if (
                "plugins" in filename
                or module_name.startswith("plugins.")
                or (
                    hasattr(frame, "f_locals")
                    and "self" in frame.f_locals
                    and hasattr(frame.f_locals["self"], "__class__")
                    and "Plugin" in frame.f_locals["self"].__class__.__name__
                )
            ):
                source = "Plugins.Unknown"
                # 缓存结果
                self._source_cache[code_id] = source
                return source

            # 默认为系统日志
            source = "OpenGewe"
            # 缓存结果
            self._source_cache[code_id] = source
            return source
        finally:
            del frame  # 避免循环引用

    def __getattr__(self, name):
        # 获取原始logger的属性
        attr = getattr(self._original_logger, name)

        # 如果是日志级别方法，包装它以添加source绑定
        if name in [
            "trace",
            "debug",
            "info",
            "success",
            "warning",
            "error",
            "critical",
        ]:

            @wraps(attr)
            def wrapped_log_method(*args, **kwargs):
                plugin_name = self._detect_plugin_name()
                context = {"source": plugin_name}

                # 添加请求ID（如果存在）
                request_id = RequestContext.get_request_id()
                if request_id:
                    context["request_id"] = request_id

                return self._original_logger.bind(**context).log(
                    name.upper(), *args, **kwargs
                )

            return wrapped_log_method

        # 对bind方法特殊处理，确保source被正确设置
        if name == "bind":

            @wraps(attr)
            def wrapped_bind(*args, **kwargs):
                if "source" not in kwargs:
                    plugin_name = self._detect_plugin_name()
                    kwargs["source"] = plugin_name

                # 添加请求ID（如果存在且未指定）
                if "request_id" not in kwargs:
                    request_id = RequestContext.get_request_id()
                    if request_id:
                        kwargs["request_id"] = request_id

                return attr(*args, **kwargs)

            return wrapped_bind

        # 对log方法特殊处理
        if name == "log":

            @wraps(attr)
            def wrapped_log(level, *args, **kwargs):
                plugin_name = self._detect_plugin_name()
                context = {"source": plugin_name}

                # 添加请求ID（如果存在）
                request_id = RequestContext.get_request_id()
                if request_id:
                    context["request_id"] = request_id

                return self._original_logger.bind(**context).log(level, *args, **kwargs)

            return wrapped_log

        # 返回原始属性
        return attr


# 全局变量，用于跟踪是否已应用拦截
_loguru_intercepted = False


def intercept_plugin_loguru():
    """拦截插件对loguru的使用，自动添加插件来源标识

    这个函数通过替换sys.modules中的loguru模块和logger对象，确保插件导入loguru时
    获取到我们的自定义版本，从而在日志记录时自动添加插件来源。
    """
    global _loguru_intercepted

    # 如果已经被拦截，不要重复操作
    if _loguru_intercepted:
        # 检查替换是否有效
        if hasattr(sys.modules.get("loguru", {}), "logger"):
            if isinstance(sys.modules["loguru"].logger, PluginLoggerProxy):
                return

    # 保存原始loguru模块和logger对象
    _original_loguru = None
    _original_logger = None

    # 尝试获取原始模块和logger
    try:
        import loguru

        _original_loguru = loguru
        _original_logger = loguru.logger
    except ImportError:
        # 如果loguru未安装，尝试从sys.modules获取
        _original_loguru = sys.modules.get("loguru")
        if _original_loguru:
            _original_logger = getattr(_original_loguru, "logger", None)

    # 如果无法获取原始对象，无法进行拦截
    if not _original_loguru or not _original_logger:
        print("警告: 无法获取原始loguru模块或logger对象，插件日志拦截将不会生效")
        return

    # 创建一个新的loguru模块，包装原始logger
    class CustomLoguru(ModuleType):
        """自定义loguru模块，代替原始模块"""

        def __init__(self):
            super().__init__("loguru")
            # 复制原始loguru模块的所有属性
            self.__dict__.update(_original_loguru.__dict__)
            # 使用代理替换logger
            self.logger = PluginLoggerProxy(_original_logger)

    # 创建我们的自定义loguru模块
    custom_loguru = CustomLoguru()

    # 替换sys.modules中的loguru
    sys.modules["loguru"] = custom_loguru

    # 直接替换原始模块的logger属性，确保已经导入的引用也被更新
    if _original_loguru and hasattr(_original_loguru, "logger"):
        _original_loguru.logger = custom_loguru.logger

    # 将修改后的logger对象挂载到全局变量中，确保即使有代码直接从logger模块导入，也能拦截到
    if "logger" in sys.modules:
        try:
            sys.modules["logger"] = custom_loguru
        except Exception:
            pass

    # 标记已拦截
    _loguru_intercepted = True

    # 设置钩子，在导入模块时自动处理
    original_import = __import__

    def patched_import(name, globals=None, locals=None, fromlist=(), level=0):
        module = original_import(name, globals, locals, fromlist, level)
        # 如果是尝试导入loguru，确保返回我们的自定义模块
        if name == "loguru" and not isinstance(module, CustomLoguru):
            return custom_loguru
        return module

    try:
        builtins = original_import("builtins")
        builtins.__import__ = patched_import
    except Exception:
        pass

    return custom_loguru.logger  # 返回代理logger对象，以便在需要时使用



"""消息处理器包

此包包含各种消息类型的处理器，用于解析和处理微信消息。
"""

# 导入基础处理器
from opengewe.callback.handlers.base import BaseHandler

# 导入文本消息处理器
from opengewe.callback.handlers.text import (
    TextMessageHandler,
    QuoteHandler,
)

# 导入媒体消息处理器
from opengewe.callback.handlers.media import (
    ImageMessageHandler,
    VoiceMessageHandler,
    VideoMessageHandler,
    EmojiMessageHandler,
)

# 导入联系人相关处理器
from opengewe.callback.handlers.contact import (
    CardHandler,
    FriendRequestHandler,
    ContactUpdateHandler,
    ContactDeletedHandler,
)

# 导入文件相关处理器
from opengewe.callback.handlers.file import (
    FileNoticeMessageHandler,
    FileMessageHandler,
)

# 导入链接相关处理器
from opengewe.callback.handlers.link import (
    LinkMessageHandler,
    FinderHandler,
    MiniappHandler,
)

# 导入位置消息处理器
from opengewe.callback.handlers.location import LocationMessageHandler

# 导入群聊消息处理器
from opengewe.callback.handlers.group import (
    GroupInviteMessageHandler,
    GroupInvitedMessageHandler,
    GroupInfoUpdateHandler,
    GroupTodoHandler,
    GroupRemovedMessageHandler,
    GroupKickMessageHandler,
    GroupDismissMessageHandler,
)

# 导入系统消息处理器
from opengewe.callback.handlers.system import (
    SysmsgHandler,
    OfflineHandler,
    SyncHandler,
)

# 导入支付相关处理器
from opengewe.callback.handlers.payment import (
    TransferHandler,
    RedPacketHandler,
)

# 默认处理器列表
DEFAULT_HANDLERS = [
    TextMessageHandler,
    ImageMessageHandler,
    VoiceMessageHandler,
    VideoMessageHandler,
    LocationMessageHandler,
    CardHandler,
    EmojiMessageHandler,
    LinkMessageHandler,
    FileNoticeMessageHandler,
    FileMessageHandler,
    ContactUpdateHandler,
    ContactDeletedHandler,
    FriendRequestHandler,
    MiniappHandler,
    QuoteHandler,
    FinderHandler,
    GroupInviteMessageHandler,
    GroupInvitedMessageHandler,
    GroupInfoUpdateHandler,
    GroupTodoHandler,
    GroupRemovedMessageHandler,
    GroupKickMessageHandler,
    GroupDismissMessageHandler,
    SysmsgHandler,
    OfflineHandler,
    TransferHandler,
    RedPacketHandler,
    SyncHandler,
]

__all__ = [
    "BaseHandler",
    "DEFAULT_HANDLERS",
]



import asyncio
import os
import base64
import tempfile
from asyncio import Future
from typing import Any, Awaitable, Callable, Dict, Optional

from celery import Celery
from celery.result import AsyncResult
from loguru import logger
import joblib

from .base import BaseMessageQueue, QueueError, WorkerNotFoundError

# 默认配置，可通过环境变量覆盖
DEFAULT_BROKER = "redis://localhost:6379/0"
DEFAULT_BACKEND = "redis://localhost:6379/0"
DEFAULT_QUEUE_NAME = "opengewe_messages"


# 创建Celery应用工厂函数
def create_celery_app(
    broker: Optional[str] = None,
    backend: Optional[str] = None,
    queue_name: Optional[str] = None,
) -> Celery:
    """创建Celery应用实例

    Args:
        broker: 消息代理的URI，例如 'redis://localhost:6379/0' 或 'amqp://guest:guest@localhost:5672//'
        backend: 结果存储的URI，例如 'redis://localhost:6379/0'
        queue_name: 队列名称

    Returns:
        Celery: Celery应用实例
    """
    # 从环境变量或参数中获取配置
    broker = broker or os.environ.get("OPENGEWE_BROKER_URL", DEFAULT_BROKER)
    backend = backend or os.environ.get("OPENGEWE_RESULT_BACKEND", DEFAULT_BACKEND)
    queue_name = queue_name or os.environ.get("OPENGEWE_QUEUE_NAME", DEFAULT_QUEUE_NAME)

    # 创建Celery应用
    app = Celery(
        "opengewe_message_queue",
        broker=broker,
        backend=backend,
    )

    # 配置Celery
    app.conf.update(
        task_serializer="json",
        accept_content=["json"],
        result_serializer="json",
        timezone="Asia/Shanghai",
        enable_utc=True,
        task_routes={"opengewe.queue.advanced.process_message": {"queue": queue_name}},
        # 添加更多Celery配置
        worker_prefetch_multiplier=1,
        task_acks_late=True,
        task_reject_on_worker_lost=True,
    )

    return app


# 创建模块级别的Celery实例
celery = create_celery_app()


# 定义处理消息的Celery任务
@celery.task(name="opengewe.queue.advanced.process_message", bind=True)
def process_message(
    self,
    task_id: str,
    serialized_func_data: str,
    func_args: list,
    func_kwargs: dict,
) -> Any:
    """处理消息的Celery任务

    Args:
        self: Celery任务实例（bind=True时自动传入）
        task_id: 任务ID
        serialized_func_data: 使用joblib序列化的函数数据
        func_args: 函数的位置参数
        func_kwargs: 函数的关键字参数

    Returns:
        Any: 函数执行的结果
    """
    try:
        logger.info(f"开始处理消息任务: {task_id}")

        # 使用joblib反序列化函数
        try:
            func_bytes = base64.b64decode(serialized_func_data)

            # 创建临时文件来存储序列化数据
            with tempfile.NamedTemporaryFile(delete=False) as temp_file:
                temp_file.write(func_bytes)
                temp_file_path = temp_file.name

            try:
                # 使用joblib加载函数
                func = joblib.load(temp_file_path)
            finally:
                # 清理临时文件
                os.unlink(temp_file_path)

        except Exception as e:
            raise ValueError(f"无法反序列化函数: {str(e)}")

        # 检查函数是否为异步函数
        if asyncio.iscoroutinefunction(func):
            # 异步函数使用asyncio.run执行
            result = asyncio.run(func(*func_args, **func_kwargs))
        else:
            # 同步函数直接调用
            result = func(*func_args, **func_kwargs)

        logger.info(f"任务 {task_id} 执行成功")
        return {"status": "success", "data": result}

    except Exception as e:
        logger.error(f"消息处理异常: {str(e)}")
        # 记录更详细的错误信息
        import traceback

        logger.error(f"错误堆栈: {traceback.format_exc()}")
        return {"status": "error", "error": str(e), "traceback": traceback.format_exc()}


class AdvancedMessageQueue(BaseMessageQueue):
    """基于Celery的高级消息队列实现"""

    def __init__(
        self,
        broker: str = DEFAULT_BROKER,
        backend: str = DEFAULT_BACKEND,
        queue_name: str = DEFAULT_QUEUE_NAME,
        celery_app: Optional[Celery] = None,
    ):
        """初始化高级消息队列

        Args:
            broker: 消息代理的URI，例如 'redis://localhost:6379/0' 或 'amqp://guest:guest@localhost:5672//'
            backend: 结果存储的URI，例如 'redis://localhost:6379/0'
            queue_name: 队列名称
            celery_app: 可选的Celery应用实例，如果提供则使用该实例，否则使用默认实例
        """
        self._futures: Dict[str, Future] = {}
        self._queue_name = queue_name
        self._processed_messages = 0
        self._is_processing = False

        # 使用提供的Celery实例或模块级别的实例
        if celery_app:
            self.celery = celery_app
        else:
            # 如果broker或backend与默认值不同，创建新的Celery实例
            if (
                broker != DEFAULT_BROKER
                or backend != DEFAULT_BACKEND
                or queue_name != DEFAULT_QUEUE_NAME
            ):
                self.celery = create_celery_app(broker, backend, queue_name)
            else:
                self.celery = celery

    def _serialize_function(self, func: Callable) -> str:
        """序列化函数为base64字符串

        Args:
            func: 要序列化的函数

        Returns:
            str: base64编码的序列化函数数据
        """
        try:
            # 创建临时文件来存储序列化数据
            with tempfile.NamedTemporaryFile(delete=False) as temp_file:
                temp_file_path = temp_file.name

            try:
                # 使用joblib序列化函数，使用默认压缩提高性能
                joblib.dump(func, temp_file_path, compress="lz4")

                # 读取序列化数据并编码为base64
                with open(temp_file_path, "rb") as f:
                    func_bytes = f.read()

                return base64.b64encode(func_bytes).decode("utf-8")

            finally:
                # 清理临时文件
                os.unlink(temp_file_path)

        except Exception as e:
            raise ValueError(f"无法序列化函数 {func.__name__}: {str(e)}")

    async def _check_workers_available(self, timeout: float = 1.0) -> tuple[bool, int]:
        """检查是否有可用的Celery worker

        Args:
            timeout: 检测超时时间，默认1秒

        Returns:
            tuple[bool, int]: (是否有可用worker, worker数量)
        """
        try:
            # 使用ping检测worker
            ping_result = self.celery.control.ping(timeout=timeout)
            if ping_result:
                worker_count = len(ping_result)
                logger.debug(f"检测到 {worker_count} 个活跃的Celery worker")
                return True, worker_count
            else:
                logger.warning("未检测到活跃的Celery worker")
                return False, 0
        except Exception as e:
            logger.error(f"检测Celery worker失败: {e}")
            return False, 0

    @property
    def is_processing(self) -> bool:
        """返回当前是否正在处理消息

        Returns:
            bool: 如果处理器正在运行则返回True，否则返回False
        """
        return self._is_processing or len(self._futures) > 0

    async def get_queue_status(self) -> Dict[str, Any]:
        """获取队列状态信息

        Returns:
            Dict[str, Any]: 包含队列当前状态的字典
        """
        try:
            # 获取Celery inspect对象
            inspect_obj = self.celery.control.inspect()

            # 获取活跃任务
            active_tasks = inspect_obj.active() or {}

            # 获取预约任务
            scheduled_tasks = inspect_obj.scheduled() or {}

            # 获取保留任务
            reserved_tasks = inspect_obj.reserved() or {}

            # 计算总任务数
            total_active = sum(len(tasks) for tasks in active_tasks.values())
            total_scheduled = sum(len(tasks) for tasks in scheduled_tasks.values())
            total_reserved = sum(len(tasks) for tasks in reserved_tasks.values())

            # 获取worker状态
            worker_stats = inspect_obj.stats() or {}
            worker_count = len(worker_stats)

            return {
                "queue_size": total_scheduled + total_reserved,
                "processing": self.is_processing,
                "worker_count": worker_count,
                "processed_messages": self._processed_messages,
                "active_tasks": total_active,
                "scheduled_tasks": total_scheduled,
                "reserved_tasks": total_reserved,
                "pending_futures": len(self._futures),
                "queue_name": self._queue_name,
                "workers": list(worker_stats.keys()),
            }
        except Exception as e:
            logger.warning(f"获取队列状态失败: {e}")
            return {
                "queue_size": 0,
                "processing": self.is_processing,
                "worker_count": 0,
                "processed_messages": self._processed_messages,
                "active_tasks": 0,
                "scheduled_tasks": 0,
                "reserved_tasks": 0,
                "pending_futures": len(self._futures),
                "queue_name": self._queue_name,
                "workers": [],
                "error": str(e),
            }

    async def clear_queue(self) -> int:
        """清空当前队列中的所有待处理消息

        Returns:
            int: 被清除的消息数量

        Raises:
            QueueError: 清空队列失败时
        """
        try:
            # 获取Celery inspect对象
            inspect_obj = self.celery.control.inspect()

            # 获取预约和保留的任务
            scheduled_tasks = inspect_obj.scheduled() or {}
            reserved_tasks = inspect_obj.reserved() or {}

            # 计算待清除的任务数
            scheduled_count = sum(len(tasks) for tasks in scheduled_tasks.values())
            reserved_count = sum(len(tasks) for tasks in reserved_tasks.values())
            total_count = scheduled_count + reserved_count

            # 清空队列
            self.celery.control.purge()

            # 取消所有待处理的Future
            cancelled_futures = 0
            for future in list(self._futures.values()):
                if not future.done():
                    future.cancel()
                    cancelled_futures += 1

            # 清空Future字典
            self._futures.clear()

            logger.info(
                f"已清空队列，删除 {total_count} 个排队任务，取消 {cancelled_futures} 个Future"
            )
            return total_count + cancelled_futures

        except Exception as e:
            error_msg = f"清空队列失败: {str(e)}"
            logger.error(error_msg)
            raise QueueError(error_msg) from e

    async def enqueue(
        self, func: Callable[..., Awaitable[Any]], *args: Any, **kwargs: Any
    ) -> Any:
        """将消息添加到队列

        Args:
            func: 要执行的异步函数
            *args: 函数的位置参数
            **kwargs: 函数的关键字参数

        Returns:
            Any: 函数执行的结果
        """
        # 检查worker可用性
        workers_available, worker_count = await self._check_workers_available()
        if not workers_available:
            error_msg = (
                "没有检测到活跃的Celery worker！\n"
                "请启动Celery worker：\n"
                f"  celery -A opengewe.queue.advanced worker --loglevel=info\n"
                "\n"
                "或者检查worker状态：\n"
                f"  celery -A opengewe.queue.advanced inspect ping\n"
                "\n"
                "确保Redis服务正在运行：\n"
                f"  {self.celery.conf.broker_url}\n"
                "\n"
                "如果您想使用简单队列，请将queue_type设置为'simple'"
            )
            raise WorkerNotFoundError(error_msg)

        logger.debug(f"检测到 {worker_count} 个可用worker，提交任务: {func.__name__}")

        # 序列化函数
        try:
            serialized_func_data = self._serialize_function(func)
        except Exception as e:
            raise QueueError(f"序列化函数失败: {str(e)}") from e

        # 创建一个Future对象用于异步等待结果
        future = Future()

        # 生成唯一的任务ID
        task_id = f"{func.__name__}_{id(future)}"

        # 存储Future以便后续设置结果
        self._futures[task_id] = future

        try:
            # 提交任务到Celery
            async_result = process_message.apply_async(
                args=(task_id, serialized_func_data, list(args), kwargs),
                task_id=task_id,
                queue=self._queue_name,  # 明确指定队列
            )

            # 创建一个监听任务结果的异步任务
            asyncio.create_task(self._wait_for_result(task_id, async_result))

            # 标记开始处理
            self._is_processing = True

            # 返回Future，等待结果
            return await future

        except Exception as e:
            # 清理Future
            self._futures.pop(task_id, None)
            raise QueueError(f"提交任务到队列失败: {str(e)}") from e

    async def _wait_for_result(
        self, task_id: str, async_result: AsyncResult, timeout: float = 30.0
    ) -> None:
        """等待Celery任务结果并设置到Future

        Args:
            task_id: 任务ID
            async_result: Celery的AsyncResult对象
            timeout: 等待超时时间，默认30秒
        """
        try:
            # 使用超时机制等待任务完成
            await asyncio.wait_for(
                self._wait_for_task_completion(async_result), timeout=timeout
            )

            # 获取任务结果
            result = async_result.result

            # 获取对应的Future
            future = self._futures.get(task_id)
            if future and not future.done():
                if isinstance(result, dict) and result.get("status") == "error":
                    # 如果任务失败，设置异常
                    error_msg = result.get("error", "Unknown error")
                    future.set_exception(Exception(error_msg))
                else:
                    # 设置结果
                    if isinstance(result, dict) and "data" in result:
                        future.set_result(result["data"])
                    else:
                        future.set_result(result)

                # 增加处理计数
                self._processed_messages += 1

        except asyncio.TimeoutError:
            # 处理超时情况
            logger.error(f"任务 {task_id} 等待超时({timeout}秒)")

            # 获取对应的Future并设置超时异常
            future = self._futures.get(task_id)
            if future and not future.done():
                timeout_msg = (
                    f"任务等待超时({timeout}秒)！可能原因：\n"
                    "1. Celery worker处理任务太慢\n"
                    "2. Worker突然停止工作\n"
                    "3. 网络连接问题\n"
                    "\n"
                    "请检查worker状态：\n"
                    f"  celery -A opengewe.queue.advanced inspect active\n"
                    f"  celery -A opengewe.queue.advanced inspect ping\n"
                    "\n"
                    f"任务ID: {task_id}"
                )
                future.set_exception(QueueError(timeout_msg))

        except Exception as e:
            logger.error(f"等待任务结果异常: {str(e)}")
            # 设置异常到Future
            future = self._futures.get(task_id)
            if future and not future.done():
                future.set_exception(e)
        finally:
            # 移除Future
            self._futures.pop(task_id, None)

            # 如果没有待处理的Future，更新处理状态
            if not self._futures:
                self._is_processing = False

    async def _wait_for_task_completion(self, async_result: AsyncResult) -> None:
        """等待任务完成的内部方法

        Args:
            async_result: Celery的AsyncResult对象
        """
        # 使用非阻塞方式等待任务完成
        while not async_result.ready():
            await asyncio.sleep(0.1)  # 短暂休眠，避免CPU占用过高

    async def start_processing(self) -> None:
        """开始处理队列中的消息

        注意：在使用Celery的情况下，消息处理是由Celery worker负责的
        此方法仅用于保持接口一致性
        """
        logger.info("Celery消息队列不需要手动启动处理，请确保Celery worker已运行")
        logger.info(f"队列名称: {self._queue_name}")

        # 检查worker状态
        try:
            inspect_obj = self.celery.control.inspect()
            worker_stats = inspect_obj.stats() or {}
            if worker_stats:
                logger.info(
                    f"发现 {len(worker_stats)} 个活跃的Celery worker: {list(worker_stats.keys())}"
                )
            else:
                logger.warning(
                    "未发现活跃的Celery worker，请启动worker: celery -A opengewe.queue.advanced worker --loglevel=info"
                )
        except Exception as e:
            logger.warning(f"无法检查Celery worker状态: {e}")

    async def stop_processing(self) -> None:
        """停止处理队列中的消息

        注意：在使用Celery的情况下，消息处理是由Celery worker负责的
        此方法仅用于保持接口一致性
        """
        logger.info("Celery消息队列不需要手动停止处理")

        # 取消所有待处理的Future
        cancelled_count = 0
        for future in list(self._futures.values()):
            if not future.done():
                future.cancel()
                cancelled_count += 1

        if cancelled_count > 0:
            logger.info(f"已取消 {cancelled_count} 个待处理的Future")

        # 清空Future字典
        self._futures.clear()
        self._is_processing = False



from dataclasses import dataclass
from typing import Dict, Any, Optional, TYPE_CHECKING
import xml.etree.ElementTree as ET

from opengewe.callback.types import MessageType
from opengewe.callback.models.base import BaseMessage

# 使用TYPE_CHECKING条件导入
if TYPE_CHECKING:
    from opengewe.client import GeweClient


@dataclass
class LinkMessage(BaseMessage):
    """链接消息"""

    title: str = ""  # 链接标题
    description: str = ""  # 链接描述
    url: str = ""  # 链接URL
    thumb_url: str = ""  # 缩略图URL
    source_username: str = ""  # 来源用户名
    source_displayname: str = ""  # 来源显示名称
    
    # 设置消息类型类变量
    message_type = MessageType.LINK
    
    async def _process_specific_data(self, data: Dict[str, Any], client: Optional["GeweClient"] = None) -> None:
        """处理链接消息特有数据"""
        # 解析XML获取链接信息
        try:
            root = ET.fromstring(self.content)
            appmsg = root.find("appmsg")
            if appmsg is not None:
                # 获取链接类型，确保是链接消息(type=5)
                link_type = appmsg.find("type")
                if link_type is not None and link_type.text == "5":
                    title = appmsg.find("title")
                    if title is not None:
                        self.title = title.text or ""

                    desc = appmsg.find("des")
                    if desc is not None:
                        self.description = desc.text or ""

                    url = appmsg.find("url")
                    if url is not None:
                        self.url = url.text or ""

                    thumb_url = appmsg.find("thumburl")
                    if thumb_url is not None:
                        self.thumb_url = thumb_url.text or ""

                    source_username = appmsg.find("sourceusername")
                    if source_username is not None:
                        self.source_username = source_username.text or ""

                    source_displayname = appmsg.find("sourcedisplayname")
                    if source_displayname is not None:
                        self.source_displayname = source_displayname.text or ""
        except Exception:
            pass


@dataclass
class MiniappMessage(BaseMessage):
    """小程序消息"""

    title: str = ""  # 小程序标题
    description: str = ""  # 小程序描述
    url: str = ""  # 小程序URL
    app_id: str = ""  # 小程序AppID
    username: str = ""  # 小程序原始ID
    pagepath: str = ""  # 小程序页面路径
    thumb_url: str = ""  # 缩略图URL
    icon_url: str = ""  # 小程序图标URL
    version: str = ""  # 小程序版本
    
    # 设置消息类型类变量
    message_type = MessageType.MINIAPP
    
    async def _process_specific_data(self, data: Dict[str, Any], client: Optional["GeweClient"] = None) -> None:
        """处理小程序消息特有数据"""
        # 解析XML获取小程序信息
        try:
            root = ET.fromstring(self.content)
            appmsg = root.find("appmsg")
            if appmsg is not None:
                # 获取小程序类型，确保是小程序消息(type=33)
                app_type = appmsg.find("type")
                if app_type is not None and app_type.text == "33":
                    title = appmsg.find("title")
                    if title is not None:
                        self.title = title.text or ""

                    desc = appmsg.find("des")
                    if desc is not None:
                        self.description = desc.text or ""

                    url = appmsg.find("url")
                    if url is not None:
                        self.url = url.text or ""

                    # 获取小程序信息
                    weappinfo = appmsg.find("weappinfo")
                    if weappinfo is not None:
                        app_id_node = weappinfo.find("appid")
                        if app_id_node is not None:
                            self.app_id = app_id_node.text or ""

                        username_node = weappinfo.find("username")
                        if username_node is not None:
                            self.username = username_node.text or ""

                        pagepath_node = weappinfo.find("pagepath")
                        if pagepath_node is not None:
                            self.pagepath = pagepath_node.text or ""

                        version_node = weappinfo.find("version")
                        if version_node is not None:
                            self.version = version_node.text or ""

                        icon_url_node = weappinfo.find("weappiconurl")
                        if icon_url_node is not None:
                            self.icon_url = icon_url_node.text or ""
        except Exception:
            pass


@dataclass
class FinderMessage(BaseMessage):
    """视频号消息"""

    finder_id: str = ""  # 视频号ID
    finder_username: str = ""  # 视频号用户名
    finder_nickname: str = ""  # 视频号昵称
    object_id: str = ""  # 内容ID
    object_type: str = ""  # 内容类型，例如视频、直播等
    object_title: str = ""  # 内容标题
    object_desc: str = ""  # 内容描述
    cover_url: str = ""  # 封面URL
    url: str = ""  # 分享链接URL
    
    # 设置消息类型类变量
    message_type = MessageType.FINDER
    
    async def _process_specific_data(self, data: Dict[str, Any], client: Optional["GeweClient"] = None) -> None:
        """处理视频号消息特有数据"""
        # 解析XML获取视频号信息
        try:
            root = ET.fromstring(self.content)
            appmsg = root.find("appmsg")
            if appmsg is not None:
                # 获取视频号ID
                finder_info = appmsg.find("finderFeed")
                if finder_info is not None:
                    self.finder_id = finder_info.get("id", "")
                    self.finder_username = finder_info.get("username", "")
                    self.finder_nickname = finder_info.get("nickname", "")
                    self.object_id = finder_info.get("objectId", "")
                    self.object_type = finder_info.get("objectType", "")
                    self.object_title = finder_info.get("title", "")
                    self.object_desc = finder_info.get("desc", "")
                    self.cover_url = finder_info.get("coverUrl", "")

                # 获取URL
                url_node = appmsg.find("url")
                if url_node is not None and url_node.text:
                    self.url = url_node.text
        except Exception:
            pass



"""消息模型模块

此模块提供了各种微信消息类型的模型类，用于统一处理和表示微信消息。
"""

# 导入基础消息类和抽象类
from opengewe.callback.models.base import (
    BaseMessage, 
    MediaBaseMessage,
    TextBaseMessage,
    FileBaseMessage,
    GroupBaseMessage,
    ContactBaseMessage,
    SystemBaseMessage,
    PaymentBaseMessage
)

# 导入文本相关消息类
from opengewe.callback.models.text import TextMessage, QuoteMessage

# 导入媒体相关消息类
from opengewe.callback.models.media import (
    ImageMessage,
    VoiceMessage,
    VideoMessage,
    EmojiMessage,
)

# 导入链接相关消息类
from opengewe.callback.models.link import (
    LinkMessage,
    MiniappMessage,
    FinderMessage,
)

# 导入文件相关消息类
from opengewe.callback.models.file import (
    FileNoticeMessage,
    FileMessage,
)

# 导入位置相关消息类
from opengewe.callback.models.location import LocationMessage

# 导入联系人相关消息类
from opengewe.callback.models.contact import (
    CardMessage,
    FriendRequestMessage,
    ContactUpdateMessage,
    ContactDeletedMessage,
)

# 导入群聊相关消息类
from opengewe.callback.models.group import (
    GroupInviteMessage,
    GroupInvitedMessage,
    GroupRemovedMessage,
    GroupKickMessage,
    GroupDismissMessage,
    GroupRenameMessage,
    GroupOwnerChangeMessage,
    GroupInfoUpdateMessage,
    GroupAnnouncementMessage,
    GroupTodoMessage,
    GroupQuitMessage,
)

# 导入系统相关消息类
from opengewe.callback.models.system import (
    RevokeMessage,
    PatMessage,
    OfflineMessage,
    SyncMessage,
)

# 导入支付相关消息类
from opengewe.callback.models.payment import (
    TransferMessage,
    RedPacketMessage,
)

# 为保证向后兼容，创建同步版本的from_dict方法
import asyncio
import inspect
import functools
from typing import Dict, Any, Optional, Type, TypeVar, get_type_hints

# 获取消息类列表
_MESSAGE_CLASSES = []
for _name, _cls in globals().copy().items():
    if (
        isinstance(_cls, type) 
        and issubclass(_cls, BaseMessage) 
        and _cls is not BaseMessage
        and not _name.endswith("BaseMessage")  # 排除抽象基类
    ):
        _MESSAGE_CLASSES.append(_cls)

# 创建同步兼容包装器
def _create_sync_from_dict(async_method):
    @functools.wraps(async_method)
    def sync_wrapper(cls, data, client=None):
        """同步版本的from_dict方法，用于向后兼容"""
        try:
            loop = asyncio.get_event_loop()
        except RuntimeError:
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)
        
        return loop.run_until_complete(async_method(cls, data, client))
    
    return classmethod(sync_wrapper)

# 为所有消息类创建同步from_dict方法
for _cls in _MESSAGE_CLASSES:
    # 确保类使用了新的异步from_dict
    if hasattr(_cls, "from_dict") and inspect.iscoroutinefunction(_cls.from_dict.__func__):
        # 类已经有异步from_dict，不需处理
        continue
    
    # 如果类定义了旧的同步from_dict，保存它
    original_from_dict = getattr(_cls, "from_dict", None)
    if original_from_dict and not inspect.iscoroutinefunction(getattr(original_from_dict, "__func__", lambda: None)):
        # 保存原始同步方法为_original_from_dict
        setattr(_cls, "_original_from_dict", original_from_dict)
    
    # 创建同步包装器
    _cls.from_dict = _create_sync_from_dict(BaseMessage.from_dict)

# 清理临时变量，避免污染命名空间
del _name, _cls, _MESSAGE_CLASSES, _create_sync_from_dict, inspect, asyncio, functools

# 导出所有消息类型
__all__ = [
    "BaseMessage",
    "MediaBaseMessage",
    "TextBaseMessage",
    "FileBaseMessage",
    "GroupBaseMessage",
    "ContactBaseMessage",
    "SystemBaseMessage",
    "PaymentBaseMessage",
    "TextMessage",
    "QuoteMessage",
    "ImageMessage",
    "VoiceMessage",
    "VideoMessage",
    "EmojiMessage",
    "LinkMessage",
    "MiniappMessage",
    "FileNoticeMessage",
    "FileMessage",
    "LocationMessage",
    "CardMessage",
    "FriendRequestMessage",
    "ContactUpdateMessage",
    "ContactDeletedMessage",
    "GroupInviteMessage",
    "GroupInvitedMessage",
    "GroupRemovedMessage",
    "GroupKickMessage",
    "GroupDismissMessage",
    "GroupRenameMessage",
    "GroupOwnerChangeMessage",
    "GroupInfoUpdateMessage",
    "GroupAnnouncementMessage",
    "GroupTodoMessage",
    "GroupQuitMessage",
    "RevokeMessage",
    "PatMessage",
    "OfflineMessage",
    "SyncMessage",
    "TransferMessage",
    "RedPacketMessage",
    "FinderMessage",
]



"""链接相关消息处理器"""

import xml.etree.ElementTree as ET
from typing import Dict, Any, Optional

from opengewe.callback.models import (
    BaseMessage,
    LinkMessage,
    FinderMessage,
    MiniappMessage,
)
from opengewe.callback.handlers.base import BaseHandler


class LinkMessageHandler(BaseHandler):
    """链接消息处理器"""

    async def can_handle(self, data: Dict[str, Any]) -> bool:
        """判断是否为链接消息"""
        if data.get("TypeName") != "AddMsg":
            return False

        if "Data" not in data:
            return False

        # 链接消息类型为49，且需要解析Content中的XML
        if data["Data"].get("MsgType") != 49:
            return False

        # 获取处理过的XML内容
        xml_content = self.extract_xml_content(data)
        if not xml_content:
            return False

        # 解析XML
        try:
            root = ET.fromstring(xml_content)
            appmsg = root.find("appmsg")
            if appmsg is not None:
                appmsg_type = appmsg.find("type")
                # 链接消息的appmsg.type为5，但需要排除群聊邀请消息
                if appmsg_type is not None and appmsg_type.text == "5":
                    title = appmsg.find("title")
                    if title is not None and "邀请你加入群聊" not in title.text:
                        return True
            return False
        except Exception:
            return False

    async def handle(self, data: Dict[str, Any]) -> Optional[BaseMessage]:
        """处理链接消息"""
        return LinkMessage.from_dict(data)


class FinderHandler(BaseHandler):
    """视频号消息处理器"""

    async def can_handle(self, data: Dict[str, Any]) -> bool:
        """判断是否为视频号消息"""
        if data.get("TypeName") != "AddMsg":
            return False

        if "Data" not in data:
            return False

        # 视频号消息类型为49
        if data["Data"].get("MsgType") != 49:
            return False

        # 获取处理过的XML内容
        xml_content = self.extract_xml_content(data)
        if not xml_content:
            return False

        try:
            root = ET.fromstring(xml_content)
            appmsg = root.find("appmsg")
            if appmsg is not None:
                # 视频号消息的类型标识为19(视频号视频分享)或22(视频号直播分享)
                type_node = appmsg.find("type")
                if type_node is not None and type_node.text in ["19", "22"]:
                    return True

                # 查找finderFeed节点，存在则为视频号消息
                finder_feed = appmsg.find("finderFeed")
                return finder_feed is not None
        except Exception:
            pass

        return False

    async def handle(self, data: Dict[str, Any]) -> Optional[BaseMessage]:
        """处理视频号消息"""
        # 直接使用FinderMessage类处理消息
        return FinderMessage.from_dict(data)


class MiniappHandler(BaseHandler):
    """小程序消息处理器"""

    async def can_handle(self, data: Dict[str, Any]) -> bool:
        """判断是否为小程序消息"""
        if data.get("TypeName") != "AddMsg":
            return False

        if "Data" not in data:
            return False

        # 小程序消息类型
        if data["Data"].get("MsgType") != 49:  # 小程序消息使用与链接相同的消息类型49
            return False

        # 获取处理过的XML内容
        xml_content = self.extract_xml_content(data)
        if not xml_content:
            return False

        try:
            root = ET.fromstring(xml_content)
            appmsg = root.find("appmsg")
            if appmsg is not None:
                # 小程序消息的类型标识为33
                type_node = appmsg.find("type")
                return type_node is not None and type_node.text == "33"
        except Exception:
            pass

        return False

    async def handle(self, data: Dict[str, Any]) -> Optional[BaseMessage]:
        """处理小程序消息"""
        # 直接使用MiniappMessage类处理消息
        return MiniappMessage.from_dict(data)



#!/usr/bin/env python3
"""
OpenGewe 模块入口点
用于直接运行 python -m opengewe 命令
"""

import os
import sys
import argparse
import asyncio


def parse_args() -> argparse.Namespace:
    """解析命令行参数

    Returns:
        argparse.Namespace: 包含解析后参数的命名空间
    """
    parser = argparse.ArgumentParser(
        description="OpenGewe 微信机器人框架",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )

    # 子命令
    subparsers = parser.add_subparsers(dest="command", help="命令")

    # 客户端子命令
    client_parser = subparsers.add_parser("client", help="启动客户端")
    client_parser.add_argument(
        "--config", type=str, default="main_config.toml", help="配置文件路径"
    )
    client_parser.add_argument("--device", type=str, default="1", help="设备ID")

    return parser.parse_args()


async def start_client(config_path: str, device_id: str) -> None:
    """启动客户端

    Args:
        config_path: 配置文件路径
        device_id: 设备ID
    """
    # 根据Python版本导入不同的TOML解析库
    try:
        import tomllib
    except ImportError:
        import tomli as tomllib
        
    from opengewe import GeweClient
    from opengewe.logger import get_logger

    logger = get_logger("GeweClient")
    logger.info(f"正在启动客户端，配置文件: {config_path}, 设备ID: {device_id}")

    try:
        with open(config_path, "rb") as f:
            config = tomllib.load(f)
    except FileNotFoundError:
        logger.error(f"配置文件不存在: {config_path}")
        sys.exit(1)
    except tomllib.TOMLDecodeError as e:
        logger.error(f"配置文件格式错误: {e}")
        sys.exit(1)

    device_config = config.get("gewe_apps", {}).get(device_id)
    if not device_config:
        logger.error(f"配置中没有找到设备: {device_id}")
        sys.exit(1)

    client = GeweClient(
        base_url=device_config.get("base_url"),
        download_url=device_config.get("download_url", ""),
        callback_url=device_config.get("callback_url", ""),
        app_id=device_config.get("app_id", ""),
        token=device_config.get("token", ""),
        is_gewe=device_config.get("is_gewe", True),
        debug=True,
    )

    try:
        logger.info("正在执行登录流程...")
        success = await client.start_login()
        if success:
            logger.info("登录成功，开始加载插件...")
            plugins_config = config.get("plugins", {})
            plugins_dir = plugins_config.get("plugins_dir", "plugins")
            loaded_plugins = await client.start_plugins(plugins_directory=plugins_dir)
            logger.info(
                f"已加载插件: {', '.join(loaded_plugins) if loaded_plugins else '无'}"
            )

            logger.info("客户端已成功启动，按 Ctrl+C 停止...")
            # 保持运行
            try:
                while True:
                    await asyncio.sleep(1)
            except KeyboardInterrupt:
                logger.info("正在关闭客户端...")
            finally:
                await client.close()
        else:
            logger.error("登录失败")
            sys.exit(1)
    except Exception as e:
        logger.error(f"运行客户端时出错: {e}")
        sys.exit(1)


def show_version() -> None:
    """显示版本信息"""
    from opengewe import __version__

    print(f"OpenGewe 版本: {__version__}")


def main() -> None:
    """主入口函数"""
    args = parse_args()

    # 设置环境变量
    if hasattr(args, "config"):
        os.environ["OPENGEWE_CONFIG"] = args.config

    # 根据子命令执行相应功能
    if args.command == "client":
        asyncio.run(start_client(args.config, args.device))
    elif args.command == "version":
        show_version()
    else:
        # 默认显示帮助信息
        print("请指定要执行的命令。使用 --help 查看帮助。")
        print("常用命令: client, version")
        sys.exit(1)


if __name__ == "__main__":
    main()



"""日志配置模块

提供基于loguru的日志配置和格式化功能。
"""

import os
import sys
import json
from typing import Dict, Optional, Any, List, Union, Callable

from loguru import logger

from opengewe.logger.formatters import (
    format_console_message,
    should_escape_message,
)

# 默认日志目录
DEFAULT_LOG_DIR = "logs"

# 日志级别对应的emoji
LEVEL_EMOJIS = {
    "TRACE": "🔍",
    "DEBUG": "🐛",
    "INFO": "ℹ️ ",
    "SUCCESS": "✅",
    "WARNING": "⚠️ ",
    "ERROR": "❌",
    "CRITICAL": "🔥",
}

# 存储当前全局日志级别
_GLOBAL_LOG_LEVEL = "INFO"

# 存储全局配置项
_LOGGER_CONFIG = {
    "console": True,
    "file": True,
    "level": "INFO",
    "format": "color",  # 可选值: "color", "simple", "json"
    "stdout": True,  # 是否将日志输出到标准输出
    "show_source": True,  # 是否显示日志源
    "show_time": True,  # 是否显示时间
}


# 获取全局日志级别
def get_global_log_level() -> str:
    """获取当前全局日志级别

    Returns:
        str: 当前设置的全局日志级别
    """
    return _GLOBAL_LOG_LEVEL


# 获取日志格式
def get_log_format() -> str:
    """获取当前日志格式

    Returns:
        str: 当前设置的日志格式
    """
    return _LOGGER_CONFIG.get("format", "color")


# 默认日志格式 - 使用居中对齐
DEFAULT_CONSOLE_FORMAT = (
    "<green>{time:YYYY-MM-DD HH:mm:ss}</green> | "
    "<level>{level.name: ^8}</level> {extra[level_emoji]} | "
    "<cyan>[{extra[source]}]</cyan> - "
    "{message}"
)

# 简单的控制台格式
SIMPLE_CONSOLE_FORMAT = (
    "{time:YYYY-MM-DD HH:mm:ss} | "
    "{level.name: ^8} | "
    "[{extra[source]}] - "
    "{message}"
)

DEFAULT_FILE_FORMAT = (
    "{time:YYYY-MM-DD HH:mm:ss} | "
    "{level.name: ^8} {extra[level_emoji]} | "
    "[{extra[source]}] | "
    "{file}:{line} - "
    "{message}"
)

# 结构化日志默认JSON格式
DEFAULT_JSON_FORMAT = {
    "time": "{time:YYYY-MM-DD HH:mm:ss}",
    "level": "{level.name}",
    "source": "{extra[source]}",
    "message": "{message}",
    "file": "{file}",
    "line": "{line}",
    "function": "{function}",
    "thread": "{thread.id}",
    "process": "{process.id}",
    "exception": "{exception}",
}

# 默认处理器配置
DEFAULT_HANDLERS = [
    # 添加API日志文件，记录所有API请求
    {
        "sink": "logs/api_{time:YYYY-MM-DD}.log",
        "level": "INFO",
        "rotation": "1 day",
        "retention": "30 days",
        "compression": "zip",
        "format": "{time:YYYY-MM-DD HH:mm:ss} | {level.name: ^8} {extra[level_emoji]} | [{extra[source]}] | {message}",
        "filter": lambda record: "API" in record["extra"].get("source", "") 
                              or "api" in record["message"].lower()
                              or "请求" in record["message"]
                              or "响应" in record["message"],
    },
    # 添加DEBUG级别日志文件，记录详细信息
    {
        "sink": "logs/debug_{time:YYYY-MM-DD}.log",
        "level": "DEBUG",
        "rotation": "1 day",
        "retention": "30 days",
        "compression": "zip",
        "format": "{time:YYYY-MM-DD HH:mm:ss} | {level.name: ^8} {extra[level_emoji]} | [{extra[source]}] | {file}:{line} - {message}",
    },
    # 添加特殊日志文件，专门记录调度任务相关信息
    {
        "sink": "logs/scheduler_{time:YYYY-MM-DD}.log",
        "level": "DEBUG",
        "rotation": "1 day",
        "retention": "30 days",
        "compression": "zip",
        "format": "{time:YYYY-MM-DD HH:mm:ss} | {level.name: ^8} {extra[level_emoji]} | [{extra[source]}] | {message}",
        "filter": lambda record: "scheduler" in record["message"].lower()
        or "task" in record["message"].lower()
        or "job" in record["message"].lower()
        or "Scheduler" in record["extra"].get("source", ""),
    },
    # 添加错误日志文件
    {
        "sink": "logs/error_{time:YYYY-MM-DD}.log",
        "level": "ERROR",
        "rotation": "1 day",
        "retention": "30 days",
        "compression": "zip",
        "format": "{time:YYYY-MM-DD HH:mm:ss} | {level.name: ^8} {extra[level_emoji]} | [{extra[source]}] | {file}:{line} - {message}",
        "backtrace": True,
        "diagnose": True,
    },
    # 添加插件日志文件
    {
        "sink": "logs/plugins_{time:YYYY-MM-DD}.log",
        "level": "DEBUG",
        "rotation": "1 day",
        "retention": "30 days",
        "compression": "zip",
        "format": "{time:YYYY-MM-DD HH:mm:ss} | {level.name: ^8} {extra[level_emoji]} | [{extra[source]}] | {message}",
        "filter": lambda record: "Plugin" in record["extra"].get("source", "") 
                              or record["extra"].get("source", "").startswith("Plugins."),
    },
]


def create_batching_sink(sink, batch_size, flush_interval):
    """创建批处理接收器

    为了避免循环导入问题，动态导入 BatchingSink

    Args:
        sink: 接收器
        batch_size: 批处理大小
        flush_interval: 刷新间隔

    Returns:
        批处理接收器
    """
    from opengewe.logger.utils import BatchingSink

    return BatchingSink(sink, batch_size, flush_interval)


def configure_from_dict(config: Dict[str, Any]) -> None:
    """从字典配置日志系统

    Args:
        config: 配置字典，包含日志相关配置项
    """
    # 更新全局配置
    global _LOGGER_CONFIG
    _LOGGER_CONFIG.update({k: v for k, v in config.items() if k in _LOGGER_CONFIG})
    
    # 提取主要配置项
    level = config.get("level", "INFO")
    format_type = config.get("format", "color")
    log_dir = config.get("path", DEFAULT_LOG_DIR)
    console = config.get("stdout", True)
    
    # 解析日志轮转参数
    rotation = config.get("rotation", "1 day")
    retention = config.get("retention", "30 days")
    compression = config.get("compression", "zip")
    
    # 根据format_type选择格式
    if format_type == "json":
        structured = True
        console_format = lambda record: format_structured_record(record, DEFAULT_JSON_FORMAT)
    elif format_type == "simple":
        structured = False
        console_format = SIMPLE_CONSOLE_FORMAT
    else:  # 默认彩色格式
        structured = False
        console_format = DEFAULT_CONSOLE_FORMAT
    
    # 配置日志系统
    setup_logger(
        console=console,
        file=True,
        level=level,
        log_dir=log_dir,
        rotation=rotation,
        retention=retention,
        compression=compression,
        console_format=console_format,
        structured=structured,
    )


def setup_logger(
    console: bool = True,
    file: bool = True,
    level: str = "INFO",
    log_dir: str = DEFAULT_LOG_DIR,
    rotation: str = "1 day",
    retention: str = "30 days",
    compression: str = "zip",
    console_format: Union[str, Callable] = DEFAULT_CONSOLE_FORMAT,
    file_format: Union[str, Callable] = DEFAULT_FILE_FORMAT,
    backtrace: bool = True,
    diagnose: bool = True,
    enqueue: bool = True,
    workers: int = 1,  # 保留参数但不再直接传递给loguru
    queue_size: int = 1000,  # 保留参数但不再直接传递给loguru
    batch_size: int = 0,
    flush_interval: float = 0.0,
    structured: bool = False,
    json_format: Optional[Dict[str, str]] = None,
    custom_handlers: List[Dict[str, Any]] = None,
) -> None:
    """配置日志记录器

    Args:
        console: 是否输出到控制台
        file: 是否记录到文件
        level: 日志级别，默认INFO
        log_dir: 日志文件目录
        rotation: 日志轮换策略
        retention: 日志保留时间
        compression: 日志压缩方式
        console_format: 控制台输出格式
        file_format: 文件记录格式
        backtrace: 是否显示回溯信息
        diagnose: 是否显示诊断信息
        enqueue: 是否启用多进程安全的异步写入
        workers: 异步处理的工作线程数，已弃用，保留参数以兼容旧代码
        queue_size: 异步队列大小，已弃用，保留参数以兼容旧代码
        batch_size: 批处理大小，设为0禁用批处理
        flush_interval: 批处理刷新间隔(秒)，设为0禁用自动刷新
        structured: 是否启用结构化日志(JSON格式)
        json_format: 自定义JSON格式，仅当structured=True时有效
        custom_handlers: 自定义处理器配置列表，如果为None则使用默认处理器
    """
    # 设置全局日志级别
    global _GLOBAL_LOG_LEVEL
    _GLOBAL_LOG_LEVEL = level.upper()

    # 更新formatters模块中的全局日志级别变量
    import opengewe.logger.formatters

    opengewe.logger.formatters.GLOBAL_LOG_LEVEL = level.upper()

    # 重置当前记录器
    logger.remove()

    # 确保日志目录存在
    if file and not os.path.exists(log_dir):
        os.makedirs(log_dir, exist_ok=True)

    # 配置通用选项
    logger.configure(
        handlers=[],
        extra={
            "source": "OpenGewe",
            "level_emoji": "",
            "request_id": None,
        },  # 默认源标识、emoji和request_id
        patcher=lambda record: record.update(
            extra={
                **record["extra"],
                "level_emoji": LEVEL_EMOJIS.get(record["level"].name, ""),
            },
            message=format_console_message(record)
            if should_escape_message(record["message"])
            else record["message"],
        ),
    )

    # 添加控制台处理器
    if console:
        # 使用结构化日志格式（如果启用）
        actual_format = console_format
        if structured and not callable(actual_format):
            def actual_format(record):
                return format_structured_record(
                    record, json_format or DEFAULT_JSON_FORMAT
                )

        # 如果需要批处理，使用BatchingSink包装sink
        if batch_size > 0:
            sink = create_batching_sink(sys.stderr, batch_size, flush_interval)
        else:
            sink = sys.stderr

        logger.add(
            sink,
            level=level,
            format=actual_format,
            colorize=True,
            backtrace=backtrace,
            diagnose=diagnose,
            enqueue=enqueue,
        )

    # 添加文件处理器 - 常规日志
    if file:
        # 使用结构化日志格式（如果启用）
        actual_format = file_format
        if structured and not callable(actual_format):
            def actual_format(record):
                return format_structured_record(
                    record, json_format or DEFAULT_JSON_FORMAT
                )

        # 常规日志文件路径
        log_file = os.path.join(log_dir, "opengewe_{time:YYYY-MM-DD}.log")

        # 如果需要批处理，使用BatchingSink
        if batch_size > 0:
            # 创建一个带有批处理的接收器
            logger.add(
                create_batching_sink(log_file, batch_size, flush_interval),
                level=level,
                format=actual_format,
                rotation=rotation,
                retention=retention,
                compression=compression,
                backtrace=backtrace,
                diagnose=diagnose,
                enqueue=enqueue,
            )
        else:
            # 直接添加标准文件接收器
            logger.add(
                log_file,
                level=level,
                format=actual_format,
                rotation=rotation,
                retention=retention,
                compression=compression,
                backtrace=backtrace,
                diagnose=diagnose,
                enqueue=enqueue,
            )

        # 添加文件处理器 - 调试日志（更详细）
        debug_file = os.path.join(log_dir, "debug_{time:YYYY-MM-DD}.log")
        if batch_size > 0:
            logger.add(
                create_batching_sink(debug_file, batch_size, flush_interval),
                level="DEBUG",
                format=actual_format,
                rotation=rotation,
                retention=retention,
                compression=compression,
                backtrace=backtrace,
                diagnose=diagnose,
                enqueue=enqueue,
            )
        else:
            logger.add(
                debug_file,
                level="DEBUG",
                format=actual_format,
                rotation=rotation,
                retention=retention,
                compression=compression,
                backtrace=backtrace,
                diagnose=diagnose,
                enqueue=enqueue,
            )

        # 添加文件处理器 - 错误日志
        error_file = os.path.join(log_dir, "error_{time:YYYY-MM-DD}.log")
        if batch_size > 0:
            logger.add(
                create_batching_sink(error_file, batch_size, flush_interval),
                level="ERROR",
                format=actual_format,
                rotation=rotation,
                retention=retention,
                compression=compression,
                backtrace=backtrace,
                diagnose=diagnose,
                enqueue=enqueue,
            )
        else:
            logger.add(
                error_file,
                level="ERROR",
                format=actual_format,
                rotation=rotation,
                retention=retention,
                compression=compression,
                backtrace=backtrace,
                diagnose=diagnose,
                enqueue=enqueue,
            )

    # 添加自定义处理器 - 优先使用传入的自定义处理器，否则使用默认处理器
    handlers_to_add = (
        custom_handlers if custom_handlers is not None else DEFAULT_HANDLERS
    )
    if handlers_to_add:
        for handler_config in handlers_to_add:
            # 复制配置以避免修改原始配置
            config = dict(handler_config)

            # 如果启用了结构化日志且配置中未指定format
            if structured and "format" in config and not callable(config["format"]):
                config["format"] = lambda record: format_structured_record(
                    record, json_format or DEFAULT_JSON_FORMAT
                )

            # 确保日志目录存在
            sink = config.get("sink")
            if isinstance(sink, str) and sink.startswith(log_dir):
                # 创建目录
                os.makedirs(os.path.dirname(sink), exist_ok=True)

            # 添加批处理支持（如果需要）
            if batch_size > 0 and isinstance(sink, str):  # 只对文件路径应用批处理
                config["sink"] = create_batching_sink(sink, batch_size, flush_interval)

            # 确保enqueue参数一致
            if "enqueue" not in config:
                config["enqueue"] = enqueue

            logger.add(**config)


def format_structured_record(
    record: Dict[str, Any], format_dict: Dict[str, str]
) -> str:
    """将日志记录格式化为结构化的JSON字符串

    Args:
        record: 日志记录
        format_dict: 格式化字典

    Returns:
        格式化后的JSON字符串
    """
    # 深度复制格式化字典
    output = {}

    # 填充字段
    for key, format_str in format_dict.items():
        try:
            # 处理嵌套结构
            if isinstance(format_str, dict):
                output[key] = {}
                for sub_key, sub_format in format_str.items():
                    output[key][sub_key] = record.get(sub_key, "")
            else:
                # 格式化字符串
                output[key] = format_str.format(**record)
        except (KeyError, ValueError):
            output[key] = f"Error formatting {key}"

    # 添加额外信息
    for key, value in record["extra"].items():
        if key not in ["level_emoji", "source"] and value is not None:
            output[key] = value

    return json.dumps(output, ensure_ascii=False)


def get_logger(source: str = "OpenGewe", **extra_context):
    """获取带有标识的日志记录器

    Args:
        source: 日志源标识，例如模块名或插件名
        **extra_context: 额外的上下文信息

    Returns:
        配置了源标识的日志记录器
    """
    context = {"source": source}
    context.update(extra_context)
    return logger.bind(**context)

# 解析日志配置文件
def load_logging_config(config_dict: Dict[str, Any]) -> None:
    """从配置文件加载日志配置

    Args:
        config_dict: 配置字典
    """
    if "logging" in config_dict:
        logging_config = config_dict["logging"]
        configure_from_dict(logging_config)
    else:
        # 使用默认配置
        setup_logger()



"""装饰器模块

提供用于消息处理和定时任务的装饰器。兼容XYBot和XXXBot插件系统的写法。
"""

from functools import wraps
from typing import Callable, Union
import pytz
from datetime import datetime
from opengewe.logger import get_logger

from apscheduler.schedulers.asyncio import AsyncIOScheduler
from apscheduler.triggers.cron import CronTrigger
from apscheduler.triggers.interval import IntervalTrigger
from apscheduler.events import EVENT_JOB_ERROR, EVENT_JOB_EXECUTED, EVENT_JOB_MISSED

from opengewe.callback.types import MessageType

# 获取装饰器模块日志记录器
logger = get_logger("Decorators")

# 创建调度器实例，设置Asia/Shanghai时区
scheduler = AsyncIOScheduler(
    timezone=pytz.timezone("Asia/Shanghai"),
    job_defaults={"misfire_grace_time": 60},  # 允许任务延迟执行60秒
)


# 添加调度器事件监听
def scheduler_listener(event):
    if hasattr(event, "job_id"):
        if event.exception:
            logger.error(f"任务 {event.job_id} 执行出错: {event.exception}")
            logger.error(f"错误详情: {event.traceback}")
        else:
            # 获取 job 对象，而不是直接使用 event.job
            job = scheduler.get_job(event.job_id)
            next_run_time = job.next_run_time if job else "未知"
            logger.debug(f"任务 {event.job_id} 执行成功，下次执行时间: {next_run_time}")


# 注册事件处理
scheduler.add_listener(
    scheduler_listener, EVENT_JOB_ERROR | EVENT_JOB_EXECUTED | EVENT_JOB_MISSED
)


def schedule(
    trigger: Union[str, CronTrigger, IntervalTrigger], **trigger_args
) -> Callable:
    """
    定时任务装饰器

    例子:

    - @schedule('interval', seconds=30)
    - @schedule('cron', hour=8, minute=30, second=30)
    - @schedule('date', run_date='2024-01-01 00:00:00')

    Args:
        trigger: 触发器类型，可以是'interval'、'cron'或'date'，也可以是触发器实例
        trigger_args: 触发器参数

    Returns:
        装饰器函数
    """

    def decorator(func: Callable) -> Callable:
        job_id = f"{func.__module__}.{func.__qualname__}"

        @wraps(func)
        async def wrapper(self, *args, **kwargs):
            try:
                logger.debug(
                    f"开始执行定时任务: {job_id}，当前时间: {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}"
                )
                result = await func(self, *args, **kwargs)
                logger.debug(f"定时任务 {job_id} 执行完成")
                return result
            except Exception as e:
                logger.error(f"定时任务 {job_id} 执行出错: {e}", exc_info=True)
                raise  # 重新抛出异常，让调度器的错误处理器处理

        setattr(wrapper, "_is_scheduled", True)
        setattr(wrapper, "_schedule_trigger", trigger)
        setattr(wrapper, "_schedule_args", trigger_args)
        setattr(wrapper, "_job_id", job_id)

        return wrapper

    return decorator


def add_job_safe(
    scheduler: AsyncIOScheduler,
    job_id: str,
    func: Callable,
    client,
    trigger: Union[str, CronTrigger, IntervalTrigger],
    **trigger_args,
) -> None:
    """添加函数到定时任务中，如果存在则先删除现有的任务

    Args:
        scheduler: 调度器实例
        job_id: 任务ID
        func: 要执行的函数
        client: GeweClient实例
        trigger: 触发器类型或实例
        trigger_args: 触发器参数
    """
    try:
        # 删除可能已存在的任务
        scheduler.remove_job(job_id)
    except Exception as e:
        logger.debug(f"删除已存在的任务 {job_id} 时出错 (可能是任务不存在): {e}")

    # 使用timezone参数确保任务使用正确的时区
    if "timezone" not in trigger_args and trigger == "date":
        trigger_args["timezone"] = scheduler.timezone

    # 添加日志记录任务添加信息
    run_time_info = ""
    if trigger == "date" and "run_date" in trigger_args:
        run_time_info = f"计划执行时间: {trigger_args['run_date']}"
    elif trigger == "interval":
        interval_desc = ", ".join([f"{k}={v}" for k, v in trigger_args.items()])
        run_time_info = f"间隔: {interval_desc}"
    elif trigger == "cron":
        cron_desc = ", ".join(
            [f"{k}={v}" for k, v in trigger_args.items() if k not in ["timezone"]]
        )
        run_time_info = f"定时: {cron_desc}"

    logger.debug(f"添加定时任务: {job_id}, 触发器类型: {trigger}, {run_time_info}")

    # 添加任务
    job = scheduler.add_job(func, trigger, args=[client], id=job_id, **trigger_args)

    # 安全地获取下次执行时间
    try:
        next_run_time = (
            job.next_run_time.strftime("%Y-%m-%d %H:%M:%S%z")
            if hasattr(job, "next_run_time") and job.next_run_time
            else "未设置"
        )
        logger.debug(f"任务 {job_id} 已添加，下次执行时间: {next_run_time}")
    except Exception as e:
        logger.debug(f"任务 {job_id} 已添加，无法获取下次执行时间: {e}")


def remove_job_safe(scheduler: AsyncIOScheduler, job_id: str) -> None:
    """从定时任务中移除任务

    Args:
        scheduler: 调度器实例
        job_id: 任务ID
    """
    try:
        scheduler.remove_job(job_id)
    except Exception:
        pass


def _create_message_handler(message_type: MessageType, priority: int = 50) -> Callable:
    """创建消息处理器装饰器

    Args:
        message_type: 消息类型
        priority: 处理优先级(0-99)，数字越小优先级越高

    Returns:
        装饰器函数
    """

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", message_type)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_text_message(priority: Union[int, Callable] = 50) -> Callable:
    """文本消息装饰器

    用于处理文本消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):  # 无参数调用时
        f = priority
        setattr(f, "_message_type", MessageType.TEXT)
        setattr(f, "_priority", 50)
        return f

    # 有参数调用时
    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.TEXT)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_at_message(priority: Union[int, Callable] = 50) -> Callable:
    """被@消息装饰器

    用于处理被@消息的装饰器。通过检查message的is_at属性判断。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.TEXT)
        setattr(f, "_priority", 50)
        setattr(f, "_is_at_message", True)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.TEXT)
        setattr(func, "_priority", min(max(priority, 0), 99))
        setattr(func, "_is_at_message", True)
        return func

    return decorator


def on_image_message(priority: Union[int, Callable] = 50) -> Callable:
    """图片消息装饰器

    用于处理图片消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.IMAGE)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.IMAGE)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_voice_message(priority: Union[int, Callable] = 50) -> Callable:
    """语音消息装饰器

    用于处理语音消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.VOICE)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.VOICE)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_emoji_message(priority: Union[int, Callable] = 50) -> Callable:
    """表情消息装饰器

    用于处理表情消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.EMOJI)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.EMOJI)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_file_message(priority: Union[int, Callable] = 50) -> Callable:
    """文件消息装饰器

    用于处理文件消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.FILE)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.FILE)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_quote_message(priority: Union[int, Callable] = 50) -> Callable:
    """引用消息装饰器

    用于处理引用消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.QUOTE)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.QUOTE)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_video_message(priority: Union[int, Callable] = 50) -> Callable:
    """视频消息装饰器

    用于处理视频消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.VIDEO)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.VIDEO)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_pat_message(priority: Union[int, Callable] = 50) -> Callable:
    """拍一拍消息装饰器

    用于处理拍一拍消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.PAT)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.PAT)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_link_message(priority: Union[int, Callable] = 50) -> Callable:
    """链接消息装饰器

    用于处理链接消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.LINK)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.LINK)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_system_message(priority: Union[int, Callable] = 50) -> Callable:
    """系统消息装饰器

    用于处理各种系统消息的装饰器，包括群信息变更等。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.UNKNOWN)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.UNKNOWN)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_other_message(priority: Union[int, Callable] = 50) -> Callable:
    """任意消息装饰器

    用于处理任何类型消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", None)  # None表示处理所有类型
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", None)  # None表示处理所有类型
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


# 以下是根据MessageType枚举扩充的装饰器，无法与XYBot或XXXBot兼容，除非他们对源代码进行扩展


def on_miniapp_message(priority: Union[int, Callable] = 50) -> Callable:
    """小程序消息装饰器

    用于处理小程序消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.MINIAPP)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.MINIAPP)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_revoke_message(priority: Union[int, Callable] = 50) -> Callable:
    """撤回消息装饰器

    用于处理撤回消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.REVOKE)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.REVOKE)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_file_notice_message(priority: Union[int, Callable] = 50) -> Callable:
    """文件通知消息装饰器

    用于处理文件发送通知的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.FILE_NOTICE)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.FILE_NOTICE)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_card_message(priority: Union[int, Callable] = 50) -> Callable:
    """名片消息装饰器

    用于处理名片消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.CARD)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.CARD)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_friend_request_message(priority: Union[int, Callable] = 50) -> Callable:
    """好友请求消息装饰器

    用于处理好友添加请求通知的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.FRIEND_REQUEST)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.FRIEND_REQUEST)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_contact_update_message(priority: Union[int, Callable] = 50) -> Callable:
    """联系人更新消息装饰器

    用于处理好友通过验证及好友资料变更的通知消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.CONTACT_UPDATE)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.CONTACT_UPDATE)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_transfer_message(priority: Union[int, Callable] = 50) -> Callable:
    """转账消息装饰器

    用于处理转账消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.TRANSFER)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.TRANSFER)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_red_packet_message(priority: Union[int, Callable] = 50) -> Callable:
    """红包消息装饰器

    用于处理红包消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.RED_PACKET)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.RED_PACKET)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_finder_message(priority: Union[int, Callable] = 50) -> Callable:
    """视频号消息装饰器

    用于处理视频号消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.FINDER)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.FINDER)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_location_message(priority: Union[int, Callable] = 50) -> Callable:
    """位置消息装饰器

    用于处理地理位置消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.LOCATION)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.LOCATION)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_group_invite_message(priority: Union[int, Callable] = 50) -> Callable:
    """群聊邀请确认消息装饰器

    用于处理群聊邀请确认通知的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.GROUP_INVITE)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.GROUP_INVITE)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_group_invited_message(priority: Union[int, Callable] = 50) -> Callable:
    """被邀请入群消息装饰器

    用于处理群聊邀请的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.GROUP_INVITED)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.GROUP_INVITED)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_group_removed_message(priority: Union[int, Callable] = 50) -> Callable:
    """被移除群聊消息装饰器

    用于处理被移除群聊通知的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.GROUP_REMOVED)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.GROUP_REMOVED)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_group_kick_message(priority: Union[int, Callable] = 50) -> Callable:
    """踢出群聊消息装饰器

    用于处理踢出群聊通知的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.GROUP_KICK)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.GROUP_KICK)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_group_dismiss_message(priority: Union[int, Callable] = 50) -> Callable:
    """解散群聊消息装饰器

    用于处理解散群聊通知的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.GROUP_DISMISS)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.GROUP_DISMISS)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_group_rename_message(priority: Union[int, Callable] = 50) -> Callable:
    """群重命名消息装饰器

    用于处理修改群名称的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.GROUP_RENAME)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.GROUP_RENAME)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_group_owner_change_message(priority: Union[int, Callable] = 50) -> Callable:
    """群主变更消息装饰器

    用于处理更换群主通知的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.GROUP_OWNER_CHANGE)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.GROUP_OWNER_CHANGE)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_group_info_update_message(priority: Union[int, Callable] = 50) -> Callable:
    """群信息更新消息装饰器

    用于处理群信息变更通知的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.GROUP_INFO_UPDATE)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.GROUP_INFO_UPDATE)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_group_announcement_message(priority: Union[int, Callable] = 50) -> Callable:
    """群公告消息装饰器

    用于处理发布群公告的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.GROUP_ANNOUNCEMENT)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.GROUP_ANNOUNCEMENT)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_group_todo_message(priority: Union[int, Callable] = 50) -> Callable:
    """群待办消息装饰器

    用于处理群待办的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.GROUP_TODO)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.GROUP_TODO)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_sync_message(priority: Union[int, Callable] = 50) -> Callable:
    """同步消息装饰器

    用于处理同步消息的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.SYNC)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.SYNC)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_contact_deleted_message(priority: Union[int, Callable] = 50) -> Callable:
    """联系人删除消息装饰器

    用于处理删除好友通知的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.CONTACT_DELETED)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.CONTACT_DELETED)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_group_quit_message(priority: Union[int, Callable] = 50) -> Callable:
    """退出群聊消息装饰器

    用于处理退出群聊的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.GROUP_QUIT)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.GROUP_QUIT)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator


def on_offline_message(priority: Union[int, Callable] = 50) -> Callable:
    """掉线通知消息装饰器

    用于处理掉线通知的装饰器。

    Args:
        priority: 处理优先级(0-99)或函数，默认为50

    Returns:
        装饰器函数
    """
    if callable(priority):
        f = priority
        setattr(f, "_message_type", MessageType.OFFLINE)
        setattr(f, "_priority", 50)
        return f

    def decorator(func: Callable) -> Callable:
        setattr(func, "_message_type", MessageType.OFFLINE)
        setattr(func, "_priority", min(max(priority, 0), 99))
        return func

    return decorator



from typing import Dict, List, Union, Any


class GroupModule:
    """异步群组模块"""

    def __init__(self, client):
        self.client = client

    async def create_chatroom(self, wxids: List[str]) -> Dict[str, Any]:
        """创建微信群

        Summary:
            创建微信群时最少要选择两位微信好友

        Args:
            wxids (List[str]): 好友的wxid列表，至少2个

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {"appId": self.client.app_id, "wxids": wxids}
        return await self.client.request("/group/createChatroom", data)

    async def modify_chatroom_name(
        self, chatroom_id: str, chatroom_name: str
    ) -> Dict[str, Any]:
        """修改群名称

        Summary:
            修改完群名称后若发现手机未展示修改后的名称，可能是手机缓存未刷新，
            手机聊天框多切换几次会刷新。

        Args:
            chatroom_id (str): 群ID
            chatroom_name (str): 群名称

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {
            "appId": self.client.app_id,
            "chatroomId": chatroom_id,
            "chatroomName": chatroom_name,
        }
        return await self.client.request("/group/modifyChatroomName", data)

    async def modify_chatroom_remark(
        self, chatroom_id: str, chatroom_remark: str
    ) -> Dict[str, Any]:
        """修改群备注

        Summary:
            群备注仅自己可见。
            修改完群备注后若发现手机未展示修改后的备注，可能是手机缓存未刷新，
            手机聊天框多切换几次会刷新。

        Args:
            chatroom_id (str): 群ID
            chatroom_remark (str): 群备注

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {
            "appId": self.client.app_id,
            "chatroomId": chatroom_id,
            "chatroomRemark": chatroom_remark,
        }
        return await self.client.request("/group/modifyChatroomRemark", data)

    async def modify_chatroom_nickname_for_self(
        self, chatroom_id: str, nickname: str
    ) -> Dict[str, Any]:
        """修改我在群内的昵称

        Args:
            chatroom_id (str): 群ID
            nickname (str): 群昵称

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {
            "appId": self.client.app_id,
            "chatroomId": chatroom_id,
            "nickName": nickname,
        }
        return await self.client.request("/group/modifyChatroomNickNameForSelf", data)

    async def invite_member(
        self, chatroom_id: str, wxids: Union[List[str], str], reason: str = ""
    ) -> Dict[str, Any]:
        """邀请/添加进群

        Args:
            chatroom_id (str): 群ID
            wxids (Union[List[str], str]): 邀请进群的好友wxid，列表或者多个英文逗号分隔的字符串
            reason (str, optional): 邀请进群的说明. Defaults to "".

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        if isinstance(wxids, list):
            wxids = ",".join(wxids)

        data = {
            "appId": self.client.app_id,
            "chatroomId": chatroom_id,
            "wxids": wxids,
            "reason": reason,
        }
        return await self.client.request("/group/inviteMember", data)

    async def remove_member(
        self, chatroom_id: str, wxids: Union[List[str], str]
    ) -> Dict[str, Any]:
        """删除群成员

        Args:
            chatroom_id (str): 群ID
            wxids (Union[List[str], str]): 删除的群成员wxid，列表或者多个英文逗号分隔的字符串

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        if isinstance(wxids, list):
            wxids = ",".join(wxids)

        data = {
            "appId": self.client.app_id,
            "chatroomId": chatroom_id,
            "wxids": wxids,
        }
        return await self.client.request("/group/removeMember", data)

    async def quit_chatroom(self, chatroom_id: str) -> Dict[str, Any]:
        """退出群聊

        Args:
            chatroom_id (str): 群ID

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {"appId": self.client.app_id, "chatroomId": chatroom_id}
        return await self.client.request("/group/quitChatroom", data)

    async def disband_chatroom(self, chatroom_id: str) -> Dict[str, Any]:
        """解散群聊

        Args:
            chatroom_id (str): 群ID

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {"appId": self.client.app_id, "chatroomId": chatroom_id}
        return await self.client.request("/group/disbandChatroom", data)

    async def get_chatroom_info(self, chatroom_id: str) -> Dict[str, Any]:
        """获取群信息

        Args:
            chatroom_id (str): 群ID

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {"appId": self.client.app_id, "chatroomId": chatroom_id}
        return await self.client.request("/group/getChatroomInfo", data)

    async def get_chatroom_member_list(self, chatroom_id: str) -> Dict[str, Any]:
        """获取群成员列表

        Args:
            chatroom_id (str): 群ID

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {"appId": self.client.app_id, "chatroomId": chatroom_id}
        return await self.client.request("/group/getChatroomMemberList", data)

    async def get_chatroom_member_detail(
        self, chatroom_id: str, member_wxids: List[str]
    ) -> Dict[str, Any]:
        """获取群成员详情

        Args:
            chatroom_id (str): 群ID
            member_wxids (List[str]): 群成员wxid列表

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {
            "appId": self.client.app_id,
            "chatroomId": chatroom_id,
            "memberWxids": member_wxids,
        }
        return await self.client.request("/group/getChatroomMemberDetail", data)

    async def get_chatroom_announcement(self, chatroom_id: str) -> Dict[str, Any]:
        """获取群公告

        Args:
            chatroom_id (str): 群ID

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {"appId": self.client.app_id, "chatroomId": chatroom_id}
        return await self.client.request("/group/getChatroomAnnouncement", data)

    async def set_chatroom_announcement(
        self, chatroom_id: str, content: str
    ) -> Dict[str, Any]:
        """设置群公告

        Args:
            chatroom_id (str): 群ID
            content (str): 公告内容

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {
            "appId": self.client.app_id,
            "chatroomId": chatroom_id,
            "content": content,
        }
        return await self.client.request("/group/setChatroomAnnouncement", data)

    async def agree_join_room(
        self, chatroom_id: str, from_wxid: str, ticket: str
    ) -> Dict[str, Any]:
        """同意进群

        Args:
            chatroom_id (str): 群ID
            from_wxid (str): 邀请人wxid
            ticket (str): ticket字符串

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {
            "appId": self.client.app_id,
            "chatroomId": chatroom_id,
            "fromWxid": from_wxid,
            "ticket": ticket,
        }
        return await self.client.request("/group/agreeJoinRoom", data)

    async def add_group_member_as_friend(
        self, chatroom_id: str, member_wxid: str, content: str = ""
    ) -> Dict[str, Any]:
        """添加群成员为好友

        Args:
            chatroom_id (str): 群ID
            member_wxid (str): 群成员wxid
            content (str, optional): 验证消息. Defaults to "".

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {
            "appId": self.client.app_id,
            "chatroomId": chatroom_id,
            "memberWxid": member_wxid,
            "content": content,
        }
        return await self.client.request("/group/addGroupMemberAsFriend", data)

    async def get_chatroom_qr_code(self, chatroom_id: str) -> Dict[str, Any]:
        """获取群二维码

        Args:
            chatroom_id (str): 群ID

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {"appId": self.client.app_id, "chatroomId": chatroom_id}
        return await self.client.request("/group/getChatroomQrCode", data)

    async def save_contract_list(self, chatroom_id: str, enabled: bool) -> Dict[str, Any]:
        """群保存到通讯录

        Args:
            chatroom_id (str): 群ID
            enabled (bool): 是否保存到通讯录

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {
            "appId": self.client.app_id,
            "chatroomId": chatroom_id,
            "enabled": enabled,
        }
        return await self.client.request("/group/saveContractList", data)

    async def admin_operate(
        self, chatroom_id: str, wxid: str, option: int
    ) -> Dict[str, Any]:
        """管理员操作

        Args:
            chatroom_id (str): 群ID
            wxid (str): 群成员wxid
            option (int): 操作类型，1设为管理员，2取消管理员

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {
            "appId": self.client.app_id,
            "chatroomId": chatroom_id,
            "wxid": wxid,
            "option": option,
        }
        return await self.client.request("/group/adminOperate", data)

    async def pin_chat(self, wxid: str, option: int) -> Dict[str, Any]:
        """聊天置顶

        Args:
            wxid (str): 好友或群wxid
            option (int): 操作类型，1设置置顶，2取消置顶

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {"appId": self.client.app_id, "wxid": wxid, "option": option}
        return await self.client.request("/group/pinChat", data)

    async def set_msg_silence(self, wxid: str, option: int) -> Dict[str, Any]:
        """设置消息免打扰

        Args:
            wxid (str): 好友或群wxid
            option (int): 操作类型，1设置免打扰，2取消免打扰

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {"appId": self.client.app_id, "wxid": wxid, "option": option}
        return await self.client.request("/group/setMsgSilence", data)

    async def join_room_using_qr_code(self, url: str) -> Dict[str, Any]:
        """扫码进群

        Args:
            url (str): 群二维码链接

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {"appId": self.client.app_id, "url": url}
        return await self.client.request("/group/joinRoomUsingQRCode", data)

    async def room_access_apply_check_approve(
        self, chatroom_id: str, apply_user_list: List[Dict[str, str]], approve: bool
    ) -> Dict[str, Any]:
        """确认进群申请

        Args:
            chatroom_id (str): 群ID
            apply_user_list (List[Dict[str, str]]): 申请人列表，每项包含wxid和ticket
            approve (bool): 是否同意

        Returns:
            Dict[str, Any]: 接口返回结果
        """
        data = {
            "appId": self.client.app_id,
            "chatroomId": chatroom_id,
            "applyUserList": apply_user_list,
            "approve": approve,
        }
        return await self.client.request("/group/roomAccessApplyCheckApprove", data)



"""支付相关消息处理器"""

import xml.etree.ElementTree as ET
from typing import Dict, Any, Optional

from opengewe.callback.models import BaseMessage, TransferMessage, RedPacketMessage
from opengewe.callback.handlers.base import BaseHandler


class TransferHandler(BaseHandler):
    """转账消息处理器"""

    async def can_handle(self, data: Dict[str, Any]) -> bool:
        """判断是否为转账消息"""
        if data.get("TypeName") != "AddMsg":
            return False

        if "Data" not in data:
            return False

        # 转账消息类型为49
        if data["Data"].get("MsgType") != 49:
            return False

        # 获取内容进一步判断
        content = data["Data"].get("Content", {}).get("string", "")
        try:
            if content:
                root = ET.fromstring(content)
                appmsg = root.find("appmsg")
                if appmsg is not None:
                    # 转账消息的类型标识为2000
                    type_node = appmsg.find("type")
                    return type_node is not None and type_node.text == "2000"
        except Exception:
            pass

        return False

    async def handle(self, data: Dict[str, Any]) -> Optional[BaseMessage]:
        """处理转账消息"""
        # 直接使用TransferMessage类处理消息
        return TransferMessage.from_dict(data)


class RedPacketHandler(BaseHandler):
    """红包消息处理器"""

    async def can_handle(self, data: Dict[str, Any]) -> bool:
        """判断是否为红包消息"""
        if data.get("TypeName") != "AddMsg":
            return False

        if "Data" not in data:
            return False

        # 红包消息类型为49
        if data["Data"].get("MsgType") != 49:
            return False

        # 获取内容进一步判断
        content = data["Data"].get("Content", {}).get("string", "")
        try:
            if content:
                root = ET.fromstring(content)
                appmsg = root.find("appmsg")
                if appmsg is not None:
                    # 红包消息的类型标识为2001(普通红包)或2002(群红包)
                    type_node = appmsg.find("type")
                    return type_node is not None and type_node.text in ["2001", "2002"]
        except Exception:
            pass

        return False

    async def handle(self, data: Dict[str, Any]) -> Optional[BaseMessage]:
        """处理红包消息"""
        # 直接使用RedPacketMessage类处理消息
        return RedPacketMessage.from_dict(data)



from dataclasses import dataclass, field
from typing import Dict, Any, Optional, TYPE_CHECKING
import xml.etree.ElementTree as ET
import re

from opengewe.callback.types import MessageType
from opengewe.callback.models.base import ContactBaseMessage, BaseMessage

if TYPE_CHECKING:
    from opengewe.client import GeweClient


@dataclass
class CardMessage(ContactBaseMessage):
    """名片消息"""
    alias: str = ""  # 微信号
    username: str = ""  # 用户名
    avatar_url: str = ""  # 头像URL
    province: str = ""  # 省份
    city: str = ""  # 城市
    sign: str = ""  # 个性签名
    sex: int = 0  # 性别，0未知，1男，2女
    
    # 设置消息类型类变量
    message_type = MessageType.CARD
    
    async def _process_specific_data(self, data: Dict[str, Any], client: Optional["GeweClient"] = None) -> None:
        """处理名片消息特有数据"""
        # 解析XML获取名片信息
        try:
            root = ET.fromstring(self.content)
            msg_node = root.find("msg")
            if msg_node is not None:
                # 从msg节点获取基本信息
                self.username = msg_node.get("username", "")
                self.nickname = msg_node.get("nickname", "")
                self.alias = msg_node.get("alias", "")
                self.province = msg_node.get("province", "")
                self.city = msg_node.get("city", "")
                self.sign = msg_node.get("sign", "")
                try:
                    self.sex = int(msg_node.get("sex", "0"))
                except ValueError:
                    pass

                # 获取头像URL
                img_node = msg_node.find("img")
                if img_node is not None:
                    self.avatar_url = img_node.get("url", "")
        except Exception:
            pass


@dataclass
class FriendRequestMessage(ContactBaseMessage):
    """好友添加请求消息"""
    stranger_wxid: str = ""  # 陌生人微信ID
    scene: int = 0  # 添加场景
    ticket: str = ""  # 验证票据
    content: str = ""  # 验证消息内容
    source: str = ""  # 来源
    alias: str = ""  # 微信号
    antispam_ticket: str = ""  # 反垃圾票据
    big_head_img_url: str = ""  # 大头像URL
    small_head_img_url: str = ""  # 小头像URL
    
    # 设置消息类型类变量
    message_type = MessageType.FRIEND_REQUEST
    
    async def _process_specific_data(self, data: Dict[str, Any], client: Optional["GeweClient"] = None) -> None:
        """处理好友请求特有数据"""
        try:
            root = ET.fromstring(self.content)
            # 检查消息类型 - 支持多种可能的格式
            if root.tag == "msg":
                # 方式1: 从属性获取
                attrs_map = {
                    "fromusername": "stranger_wxid",
                    "fromnickname": "nickname",
                    "content": "content",
                    "scene": "scene",
                    "ticket": "ticket",
                    "encryptusername": "ticket",
                    "sourceusername": "source",
                    "sourcenickname": "source",
                    "alias": "alias",
                    "antispamticket": "antispam_ticket",
                    "bigheadimgurl": "big_head_img_url",
                    "smallheadimgurl": "small_head_img_url",
                }

                # 处理属性
                for attr, field in attrs_map.items():
                    value = root.get(attr)
                    if value is not None and not getattr(self, field):
                        if field == "scene" and value:
                            try:
                                setattr(self, field, int(value))
                            except (ValueError, TypeError):
                                pass
                        else:
                            setattr(self, field, value)

                # 方式2: 从内部XML元素获取
                if not (self.stranger_wxid and self.nickname and self.ticket):
                    tags_map = {
                        "username": "stranger_wxid",
                        "nickname": "nickname",
                        "content": "content",
                        "alias": "alias",
                        "scene": "scene",
                        "ticket": "ticket",
                        "encryptusername": "ticket",
                        "source": "source",
                        "sourceusername": "source",
                        "antispamticket": "antispam_ticket",
                        "bigheadimgurl": "big_head_img_url",
                        "smallheadimgurl": "small_head_img_url",
                    }

                    # 搜索所有元素
                    for elem in root.findall(".//*"):
                        if elem.tag in tags_map and elem.text:
                            field = tags_map[elem.tag]
                            if not getattr(self, field):
                                if field == "scene" and elem.text:
                                    try:
                                        setattr(self, field, int(elem.text))
                                    except (ValueError, TypeError):
                                        pass
                                else:
                                    setattr(self, field, elem.text)

                # 方式3: 如果还没有找到stranger_wxid，尝试从内容中提取
                if not self.stranger_wxid:
                    # 尝试多种可能的格式
                    patterns = [
                        r'fromusername="([^"]+)"',
                        r'username="([^"]+)"',
                        r'encryptusername="([^"]+)"',
                    ]

                    for pattern in patterns:
                        match = re.search(pattern, self.content)
                        if match:
                            self.stranger_wxid = match.group(1)
                            break

                # 方式4: 检查用户名是否在self.content中直接出现
                if not self.stranger_wxid and "wxid_" in self.content:
                    match = re.search(r"(wxid_[a-zA-Z0-9_-]+)", self.content)
                    if match:
                        self.stranger_wxid = match.group(1)

                # 方式5: 从XML内容直接提取完整的用户名
                if not self.nickname and "fromnickname" in self.content:
                    match = re.search(r'fromnickname="([^"]+)"', self.content)
                    if match:
                        self.nickname = match.group(1)
        except Exception as e:
            # 记录异常信息到raw_data，便于调试
            self.raw_data["xml_parse_error"] = str(e)

        # 检查PushContent字段，可能包含发送者昵称和请求内容
        if "Data" in data:
            push_content = data["Data"].get("PushContent", "")
            if isinstance(push_content, str) and not self.nickname:
                # 格式通常为 "昵称 : [名片]姓名" 或 "昵称请求添加你为朋友"
                name_match = re.match(r"^([^:]+)(?:\s*:|请求)", push_content)
                if name_match:
                    self.nickname = name_match.group(1).strip()


@dataclass
class ContactUpdateMessage(BaseMessage):
    """联系人更新消息"""
    contact_info: Dict[str, Any] = field(default_factory=dict)  # 联系人信息
    user_type: int = 0  # 用户类型
    username: str = ""  # 用户名
    nickname: str = ""  # 昵称
    remark: str = ""  # 备注
    alias: str = ""  # 微信号
    avatar_url: str = ""  # 头像URL
    sex: int = 0  # 性别
    signature: str = ""  # 签名
    province: str = ""  # 省份
    city: str = ""  # 城市
    country: str = ""  # 国家
    is_chatroom: bool = False  # 是否为群聊
    
    # 设置消息类型类变量
    message_type = MessageType.CONTACT_UPDATE
    
    async def _process_specific_data(self, data: Dict[str, Any], client: Optional["GeweClient"] = None) -> None:
        """处理联系人更新特有数据"""
        if "Data" in data:
            msg_data = data["Data"]
            
            # ModContacts消息的结构不同，从Data中直接获取联系人信息
            if isinstance(msg_data, dict):
                # 获取基础信息
                self.username = msg_data.get("UserName", {}).get("string", "")

                # 判断是否为群聊
                self.is_chatroom = "@chatroom" in self.username

                # 设置联系人类型
                try:
                    self.user_type = int(msg_data.get("Type", 0))
                except (ValueError, TypeError):
                    pass

                # 获取昵称
                if "NickName" in msg_data and "string" in msg_data["NickName"]:
                    self.nickname = msg_data["NickName"]["string"]

                # 获取备注
                if "Remark" in msg_data and "string" in msg_data["Remark"]:
                    self.remark = msg_data["Remark"]["string"]

                # 获取微信号
                if "Alias" in msg_data and "string" in msg_data["Alias"]:
                    self.alias = msg_data["Alias"]["string"]

                # 获取签名
                if "Signature" in msg_data and "string" in msg_data["Signature"]:
                    self.signature = msg_data["Signature"]["string"]

                # 获取省份
                if "Province" in msg_data and "string" in msg_data["Province"]:
                    self.province = msg_data["Province"]["string"]

                # 获取城市
                if "City" in msg_data and "string" in msg_data["City"]:
                    self.city = msg_data["City"]["string"]

                # 获取国家
                if "Country" in msg_data and "string" in msg_data["Country"]:
                    self.country = msg_data["Country"]["string"]

                # 获取性别
                try:
                    self.sex = int(msg_data.get("Sex", 0))
                except (ValueError, TypeError):
                    pass

                # 获取头像URL
                if "HeadImgUrl" in msg_data and "string" in msg_data["HeadImgUrl"]:
                    self.avatar_url = msg_data["HeadImgUrl"]["string"]

                # 构建联系人信息字典
                self.contact_info = {
                    "username": self.username,
                    "nickname": self.nickname,
                    "remark": self.remark,
                    "alias": self.alias,
                    "signature": self.signature,
                    "province": self.province,
                    "city": self.city,
                    "country": self.country,
                    "sex": self.sex,
                    "avatar_url": self.avatar_url,
                    "is_chatroom": self.is_chatroom,
                    "user_type": self.user_type,
                }


@dataclass
class ContactDeletedMessage(BaseMessage):
    """联系人删除消息"""
    username: str = ""  # 被删除联系人的用户名
    is_chatroom: bool = False  # 是否为群聊
    
    # 设置消息类型类变量
    message_type = MessageType.CONTACT_DELETED
    
    async def _process_specific_data(self, data: Dict[str, Any], client: Optional["GeweClient"] = None) -> None:
        """处理联系人删除特有数据"""
        if "Data" in data:
            # 处理两种可能的数据结构
            if (
                isinstance(data["Data"], dict)
                and "UserName" in data["Data"]
                and "string" in data["Data"]["UserName"]
            ):
                # 结构为 {"UserName": {"string": "wxid_xxx"}}
                self.username = data["Data"]["UserName"]["string"]
            else:
                # 结构为直接的字符串或其他格式
                self.username = str(data["Data"])

            # 判断是否为群聊
            self.is_chatroom = "@chatroom" in self.username
